package com.neurosdk2;

import static com.neurosdk2.Utils.DATA_ID;
import static com.neurosdk2.Utils.GUID_ID;
import static com.neurosdk2.Utils.sendEvent;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.ISensor;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.ParameterInfo;
import com.neurosdk2.neuro.types.SensorCommand;
import com.neurosdk2.neuro.types.SensorFeature;
import com.neurosdk2.neuro.types.SensorParameter;
import com.neurosdk2.neuro.types.SensorState;
import com.neurosdk2.neuro.types.SensorVersion;

import java.util.Objects;

public abstract class AFullSensor {

  protected Sensor sensor;
  protected String guid;
  protected ReactApplicationContext reactContext;


  // <editor-fold> Utils
  protected void addCallbacks(){
    sensor.sensorStateChanged = (state)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableMap map = Arguments.createMap();
      map.putInt("state",state.index());
      result.putMap(DATA_ID, map);
      sendEvent(reactContext,Constants.CONNECTION_CHANGED_EVENT_NAME.raw(), result);
    };
    sensor.batteryChanged = (battery)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableMap map = Arguments.createMap();
      map.putInt("power",battery);
      result.putMap(DATA_ID, map);
      sendEvent(reactContext,Constants.BATTERY_CHANGED_EVENT_NAME.raw(), result);
    };
  }
  protected void removeCallbacks(){
    sensor.sensorStateChanged = null;
    sensor.batteryChanged = null;
  }
  // </editor-fold>

  // <editor-fold> Sensor
  public void closeSensor() {
    try {
      removeCallbacks();
      sensor.close();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }


  public void connectSensor() {
    sensor.connect();
  }


  public void disconnectSensor() {
    sensor.disconnect();
  }


  public void supportedFeatures(WritableArray features) {
    for (SensorFeature feature : sensor.getSupportedFeature()) {
      features.pushInt(feature.index());
    }
  }


  public boolean isSupportedFeature(int feature) {
    return sensor.isSupportedFeature(SensorFeature.indexOf(feature));
  }

  public void supportedCommands(WritableArray commands) {
    try {
      for (SensorCommand sensorCommand : sensor.getSupportedCommand()) {
        commands.pushInt(sensorCommand.index());
      }
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }


  public boolean isSupportedCommand(int command) {
    return sensor.isSupportedCommand(SensorCommand.indexOf(command));
  }


  public void supportedParameters(WritableArray params) {
    for (ParameterInfo parameterInfo : sensor.getSupportedParameter()) {
      WritableMap param = Arguments.createMap();
      param.putInt("Param", parameterInfo.getParam().index());
      param.putInt("ParamAccess", parameterInfo.getParamAccess().index());
      params.pushMap(param);
    }
  }


  public boolean isSupportedParameter(int parameter) {
    return sensor.isSupportedParameter(SensorParameter.indexOf(parameter));
  }


  public void execCommand(int command) {
    sensor.execCommand(SensorCommand.indexOf(command));

  }


  public String getName() {
    return sensor.getName();
  }


  public void setName(String name) {
    sensor.setName(name);
  }


  public int getState() {
    return sensor.getState().index();
  }


  public String getAddress() {
    return sensor.getAddress();
  }


  public String getSerialNumber() {
    return sensor.getSerialNumber();
  }


  public void setSerialNumber(String sn) {
    sensor.setSerialNumber(sn);
  }


  public int getBattPower() {
    return sensor.getBattPower();
  }


  public int getSamplingFrequency() {
    return sensor.getSamplingFrequency().index();
  }


  public int getGain() {
    return sensor.getGain().index();
  }


  public int getDataOffset() {
    return sensor.getDataOffset().index();
  }


  public int getFirmwareMode() {
    return sensor.getFirmwareMode().index();
  }


  public void getVersion(WritableMap res) {
    SensorVersion sensorVersion = sensor.getVersion();
    res.putInt("FwMajor", sensorVersion.getFwMajor());
    res.putInt("FwMinor", sensorVersion.getFwMinor());
    res.putInt("FwPatch", sensorVersion.getFwPatch());
    res.putInt("HwMajor", sensorVersion.getHwMajor());
    res.putInt("HwMinor", sensorVersion.getHwMinor());
    res.putInt("HwPatch", sensorVersion.getHwPatch());
    res.putInt("ExtMajor", sensorVersion.getExtMajor());
  }


  public int getChannelsCount() {
    return sensor.getChannelsCount();
  }


  public int getSensFamily() {
    return sensor.getSensFamily().index();
  }
  // </editor-fold>

  protected abstract String getSensorName();

  // <editor-fold> Concrete sensors


  public int getAmpMode() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getSamplingFrequencyResist() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }


  public int getSamplingFrequencyFPG() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getIrAmplitude() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setIrAmplitude(int amp) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getRedAmplitude() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setRedAmplitude(int amp) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void pingNeuroSmart(double marker) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }


  public boolean isSupportedFilter(int filter) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getSupportedFilters(WritableArray filters) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getHardwareFilters(WritableArray filters) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setHardwareFilters(ReadableArray filters) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setFirmwareMode(int mode) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setSamplingFrequency(int sf) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setGain(int gain) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setDataOffset(int offset) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getExtSwInput() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setExtSwInput(int extSwInp) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getADCInput() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setADCInput(int adcInp) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getStimulatorMAState(WritableMap res) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getStimulatorParam(WritableMap map) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setStimulatorParam(ReadableMap param) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getMotionAssistantParam(WritableMap map) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setMotionAssistantParam(ReadableMap param) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getMotionCounterParam(WritableMap map) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setMotionCounterParam(ReadableMap param) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getMotionCounter() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getColor() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public boolean getMEMSCalibrateState() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getSamplingFrequencyResp() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getSamplingFrequencyEnvelope() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getSignalType() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setSignalType(int type) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getElectrodeState() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }


  public int getAccSens() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setAccSens(int accSens) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getGyroSens() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setGyroSens(int gyroSens) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public int getSamplingFrequencyMEMS() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }



  public int getSurveyId() throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setSurveyId(int id) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void getAmplifierParam(WritableMap map) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setAmplifierParam(ReadableMap param) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }


  // </editor-fold>

  public void getSupportedChannels(WritableArray channels) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }


  public void getAmplifierParamBB2(WritableMap map) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }
  public void setAmplifierParamBB2(ReadableMap param) throws Exception {
    throw new UnsupportedOperationException("Unsupported " + getSensorName() + " parameter");
  }

}
