package com.neurosdk2;

import static com.neurosdk2.Utils.ConvertFPGData;
import static com.neurosdk2.Utils.ConvertMEMSData;
import static com.neurosdk2.Utils.ConvertReactToBoolArray;
import static com.neurosdk2.Utils.ConvertReactToIntArray;
import static com.neurosdk2.Utils.ConvertSignalChannelsData;
import static com.neurosdk2.Utils.DATA_ID;
import static com.neurosdk2.Utils.GUID_ID;
import static com.neurosdk2.Utils.sendEvent;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.BrainBit2;
import com.neurosdk2.neuro.BrainBit2;
import com.neurosdk2.neuro.NeuroEEG;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.BrainBit2AmplifierParam;
import com.neurosdk2.neuro.types.EEGChannelInfo;
import com.neurosdk2.neuro.types.FPGData;
import com.neurosdk2.neuro.types.IrAmplitude;
import com.neurosdk2.neuro.types.MEMSData;
import com.neurosdk2.neuro.types.NeuroEEGAmplifierParam;
import com.neurosdk2.neuro.types.RedAmplitude;
import com.neurosdk2.neuro.types.ResistRefChannelsData;
import com.neurosdk2.neuro.types.SignalChannelsData;

import java.util.Arrays;
import java.util.List;

public class BrainBit2Impl extends AFullSensor{

    public BrainBit2Impl(Sensor sensor, String guid, ReactApplicationContext context){
        this.sensor = sensor;
        this.guid = guid;
        this.reactContext = context;
        addCallbacks();
    }

    @Override
    protected void addCallbacks() {
        super.addCallbacks();
        ((BrainBit2)sensor).signalDataReceived = (data)->{
            WritableMap result = Arguments.createMap();
            result.putString(GUID_ID, guid);
            WritableArray signalArray = Arguments.createArray();
            for (SignalChannelsData signal : data) {
                WritableMap map = Arguments.createMap();
                ConvertSignalChannelsData(signal, map);
                signalArray.pushMap(map);
            }
            result.putArray(DATA_ID, signalArray);
            sendEvent(reactContext, Constants.BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME.raw(), result);
        };
        ((BrainBit2)sensor).resistDataReceived = (data)->{
            WritableMap result = Arguments.createMap();
            result.putString(GUID_ID, guid);
            WritableArray array = Arguments.createArray();
            for(ResistRefChannelsData resists : data){
                WritableMap map = Arguments.createMap();
                map.putInt("PackNum", resists.getPackNum());
                map.putArray("Samples",Arguments.fromArray(resists.getSamples()));
                map.putArray("Referents",Arguments.fromArray(resists.getReferents()));
                array.pushMap(map);
            }
            result.putArray(DATA_ID, array);
            sendEvent(reactContext, Constants.BRAINBIT2_RESIST_RECEIVED_EVENT_NAME.raw(), result);
        };
        ((BrainBit2)sensor).fpgDataReceived = (data)->{
            WritableMap result = Arguments.createMap();
            result.putString(GUID_ID, guid);
            WritableArray array = Arguments.createArray();
            for (FPGData fpgData : data) {
                WritableMap map = Arguments.createMap();
                ConvertFPGData(fpgData, map);
                array.pushMap(map);
            }
            result.putArray(DATA_ID, array);
            sendEvent(reactContext, Constants.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME.raw(), result);
        };
        ((BrainBit2)sensor).memsDataReceived = (data)->{
            WritableMap result = Arguments.createMap();
            result.putString(GUID_ID, guid);
            WritableArray array = Arguments.createArray();
            for (MEMSData memsData : data) {
                WritableMap map = Arguments.createMap();
                ConvertMEMSData(memsData, map);
                array.pushMap(map);
            }
            result.putArray(DATA_ID, array);
            sendEvent(reactContext,Constants.MEMSDATA_CHANGED_EVENT_NAME.raw(), result);
        };
        ((BrainBit2)sensor).sensorAmpModeChanged = (data)->{
            WritableMap result = Arguments.createMap();
            result.putString(GUID_ID, guid);
            result.putInt(DATA_ID, data.index());
            sendEvent(reactContext,Constants.AMPMODE_CHANGED_EVENT_NAME.raw(), result);
        };
    }

    @Override
    protected void removeCallbacks() {
        super.removeCallbacks();
        ((BrainBit2)sensor).signalDataReceived = null;
        ((BrainBit2)sensor).resistDataReceived = null;
        ((BrainBit2)sensor).fpgDataReceived = null;
        ((BrainBit2)sensor).memsDataReceived = null;
        ((BrainBit2)sensor).sensorAmpModeChanged = null;
    }

    @Override
    protected String getSensorName() {
        return "BrainBit2";
    }

    @Override
    public int getSamplingFrequencyMEMS() throws Exception {
        return ((BrainBit2)sensor).getSamplingFrequencyMEMS().index();
    }

    @Override
    public int getSamplingFrequencyFPG() throws Exception {
        return ((BrainBit2)sensor).getSamplingFrequencyFPG().index();
    }

    @Override
    public int getSamplingFrequencyResist() throws Exception {
        return ((BrainBit2)sensor).getSamplingFrequencyResist().index();
    }

    @Override
    public int getIrAmplitude() throws Exception {
        return ((BrainBit2)sensor).getIrAmplitude().index();
    }

    @Override
    public void setIrAmplitude(int amp) throws Exception {
        ((BrainBit2)sensor).setIrAmplitude(IrAmplitude.indexOf(amp));
    }

    @Override
    public int getRedAmplitude() throws Exception {
        return ((BrainBit2)sensor).getRedAmplitude().index();
    }

    @Override
    public void setRedAmplitude(int amp) throws Exception {
        ((BrainBit2)sensor).setRedAmplitude(RedAmplitude.indexOf(amp));
    }

    @Override
    public int getAmpMode() throws Exception {
        return ((BrainBit2)sensor).getAmpMode().index();
    }

    @Override
    public void pingNeuroSmart(double marker) throws Exception {
        ((BrainBit2)sensor).pingNeuroSmart((byte)marker);
    }

    @Override
    public void getSupportedChannels(WritableArray channels) throws Exception {
        List<EEGChannelInfo> channelsList = ((BrainBit2)sensor).getSupportedChannels();
        for (EEGChannelInfo channel : channelsList) {
            WritableMap channelMap = Arguments.createMap();
            channelMap.putInt("Id",channel.getId().index());
            channelMap.putInt("ChType",channel.getChType().index());
            channelMap.putString("Name",channel.getName());
            channelMap.putInt("Num",channel.getNum());
            channels.pushMap(channelMap);
        }
    }

    @Override
    public void getAmplifierParamBB2(WritableMap map) throws Exception {
        BrainBit2AmplifierParam param = ((BrainBit2)sensor).getAmplifierParam();
        map.putInt("Current",param.getRawCurrent());
        map.putArray("ChGain",Arguments.fromArray(param.getRawChGain()));
        map.putArray("ChSignalMode",Arguments.fromArray(param.getRawChSignalMode()));
        map.putArray("ChResistUse",Arguments.fromArray(param.getChResistUse()));
    }

    @Override
    public void setAmplifierParamBB2(ReadableMap param) throws Exception {
        BrainBit2AmplifierParam oldP = ((BrainBit2)sensor).getAmplifierParam();
        if(param.getArray("ChSignalMode") != null){
            oldP = new BrainBit2AmplifierParam(ConvertReactToIntArray(param.getArray("ChSignalMode")),
                    oldP.getChResistUse(),
                    oldP.getRawChGain(),
                    oldP.getRawCurrent());
        }
        if(param.getArray("ChResistUse") != null){
            oldP = new BrainBit2AmplifierParam(
                    oldP.getRawChSignalMode(),
                    ConvertReactToBoolArray(param.getArray("ChResistUse")),
                    oldP.getRawChGain(),
                    oldP.getRawCurrent()
            );
        }
        if(param.getArray("ChGain") != null){
            oldP = new BrainBit2AmplifierParam(
                    oldP.getRawChSignalMode(),
                    oldP.getChResistUse(),
                    ConvertReactToIntArray(param.getArray("ChGain")),
                    oldP.getRawCurrent()
            );
        }
        try {
            int curr = param.getInt("Current");
            oldP = new BrainBit2AmplifierParam(
                    oldP.getRawChSignalMode(),
                    oldP.getChResistUse(),
                    oldP.getRawChGain(),
                    curr
            );
        }
        catch(Exception ex){
            ex.printStackTrace();
        }
        ((BrainBit2)sensor).setAmplifierParam(oldP);
    }
}
