package com.neurosdk2;

import static com.neurosdk2.Utils.DATA_ID;
import static com.neurosdk2.Utils.GUID_ID;
import static com.neurosdk2.Utils.sendEvent;

import android.telecom.Call;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.BrainBit;
import com.neurosdk2.neuro.Callibri;
import com.neurosdk2.neuro.ISensor;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.BrainBitSignalData;
import com.neurosdk2.neuro.types.SensorGain;

public class BrainBitImpl extends AFullSensor {

  public BrainBitImpl(Sensor sensor, String guid, ReactApplicationContext context){
    this.sensor = sensor;
    this.guid = guid;
    this.reactContext = context;
    addCallbacks();
  }

  @Override
  protected void addCallbacks() {
    super.addCallbacks();
    ((BrainBit)sensor).brainBitResistDataReceived = (data)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableMap map = Arguments.createMap();
      map.putDouble("O1", data.getO1());
      map.putDouble("O2", data.getO2());
      map.putDouble("T3", data.getT3());
      map.putDouble("T4", data.getT4());
      result.putMap(DATA_ID, map);
      sendEvent(reactContext, Constants.BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME.raw(), result);
    };
    ((BrainBit)sensor).brainBitSignalDataReceived = (data)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray samples = Arguments.createArray();
      for (BrainBitSignalData signalData : data) {
        WritableMap map = Arguments.createMap();
        map.putInt("Marker", signalData.getMarker());
        map.putInt("PackNum", signalData.getPackNum());
        map.putDouble("O1", signalData.getO1());
        map.putDouble("O2", signalData.getO2());
        map.putDouble("T3", signalData.getT3());
        map.putDouble("T4", signalData.getT4());
        samples.pushMap(map);
      }
      result.putArray(DATA_ID, samples);
      sendEvent(reactContext, Constants.BRAINBIT_SIGNAL_CHANGED_EVENT_NAME.raw(), result);
    };
  }

  @Override
  protected void removeCallbacks() {
    super.removeCallbacks();
    ((BrainBit)sensor).brainBitResistDataReceived = null;
    ((BrainBit)sensor).brainBitSignalDataReceived = null;
  }

  @Override
  protected String getSensorName() {
    return "BrainBit";
  }

  @Override
  public void setGain(int gain) throws Exception {
    ((BrainBit)sensor).setGain(SensorGain.indexOf(gain));
  }
}
