package com.neurosdk2;

import static com.neurosdk2.Utils.ConvertReactToIntArray;
import static com.neurosdk2.Utils.ConvertSignalChannelsData;
import static com.neurosdk2.Utils.DATA_ID;
import static com.neurosdk2.Utils.GUID_ID;
import static com.neurosdk2.Utils.sendEvent;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.NeuroEEG;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.EEGChannelInfo;
import com.neurosdk2.neuro.types.NeuroEEGAmplifierParam;
import com.neurosdk2.neuro.types.ResistChannelsData;
import com.neurosdk2.neuro.types.NeuroEEGFSStatus;
import com.neurosdk2.neuro.types.SensorCommand;
import com.neurosdk2.neuro.types.SensorFSStatus;
import com.neurosdk2.neuro.types.SignalChannelsData;


public class NeuroEEGImpl extends AFullSensor{

  public NeuroEEGImpl(Sensor sensor, String guid, ReactApplicationContext context){
    this.sensor = sensor;
    this.guid = guid;
    this.reactContext = context;
    NeuroEEGFSStatus fsStatus = ((NeuroEEG)sensor).getFSStatus();
    if(fsStatus.getStatus() == SensorFSStatus.OK){
      sensor.execCommand(SensorCommand.CommandFileSystemStreamClose);
      sensor.execCommand(SensorCommand.CommandFileSystemDisable);
    }
    addCallbacks();
  }

  @Override
  protected void addCallbacks() {
    super.addCallbacks();
    ((NeuroEEG)sensor).ampModeChanged = (data)->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableMap map = Arguments.createMap();
      result.putInt(DATA_ID, data.index());
      sendEvent(reactContext,Constants.AMPMODE_CHANGED_EVENT_NAME.raw(), result);
    };
    ((NeuroEEG)sensor).signalDataReceived = data -> {
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray signalArray = Arguments.createArray();
      for (SignalChannelsData signal : data) {
        WritableMap map = Arguments.createMap();
        ConvertSignalChannelsData(signal, map);
        signalArray.pushMap(map);
      }
      result.putArray(DATA_ID, signalArray);
      sendEvent(reactContext,Constants.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME.raw(), result);

    };
    ((NeuroEEG)sensor).resistDataReceived = data ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray resistArray = Arguments.createArray();
      for (ResistChannelsData resist : data) {
        WritableMap map = Arguments.createMap();
        ConvertResistChannelsData(resist, map);
        resistArray.pushMap(map);
      }
      result.putArray(DATA_ID, resistArray);
      sendEvent(reactContext,Constants.NEURRO_EEG_RESIST_DATA_EVENT_NAME.raw(), result);
    };
    ((NeuroEEG)sensor).signalResistReceived = (signalData, resistData) ->{
      WritableMap result = Arguments.createMap();
      result.putString(GUID_ID, guid);
      WritableArray signalArray = Arguments.createArray();
      for (SignalChannelsData signal : signalData) {
        WritableMap map = Arguments.createMap();
        ConvertSignalChannelsData(signal, map);
        signalArray.pushMap(map);
      }
      WritableArray resistArray = Arguments.createArray();
      for (ResistChannelsData resist : resistData) {
        WritableMap map = Arguments.createMap();
        ConvertResistChannelsData(resist, map);
        resistArray.pushMap(map);
      }
      WritableMap map = Arguments.createMap();
      map.putArray("signalData", signalArray);
      map.putArray("resistData", resistArray);
      result.putMap(DATA_ID, map);
      sendEvent(reactContext,Constants.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME.raw(), result);
    };

  }

  @Override
  protected void removeCallbacks() {
    super.removeCallbacks();
    ((NeuroEEG)sensor).ampModeChanged = null;
    ((NeuroEEG)sensor).signalDataReceived = null;
    ((NeuroEEG)sensor).resistDataReceived = null;
    ((NeuroEEG)sensor).signalResistReceived = null;

  }

  @Override
  protected String getSensorName() {
    return "NeuroEEG";
  }

  @Override
  public int getSurveyId() throws Exception {
    return ((NeuroEEG)sensor).getSurveyId();
  }

  @Override
  public void setSurveyId(int id) throws Exception {
    ((NeuroEEG)sensor).setSurveyId(id);
  }

  @Override
  public int getAmpMode() throws Exception {
    return ((NeuroEEG)sensor).getAmpMode().index();
  }

  @Override
  public void getSupportedChannels(WritableArray channels) throws Exception {
    List<EEGChannelInfo> channelsList = ((NeuroEEG)sensor).getSupportedChannels();
    for (EEGChannelInfo channel : channelsList) {
      WritableMap channelMap = Arguments.createMap();
      channelMap.putInt("Id",channel.getId().index());
      channelMap.putInt("ChType",channel.getChType().index());
      channelMap.putString("Name",channel.getName());
      channelMap.putInt("Num",channel.getNum());
      channels.pushMap(channelMap);
    }
  }

  @Override
  public void getAmplifierParam(WritableMap map) throws Exception {
    NeuroEEGAmplifierParam param = ((NeuroEEG)sensor).getAmplifierParam();
    map.putBoolean("ReferentResistMesureAllow",param.getReferentResistMesureAllow());
    map.putInt("Frequency",param.getFrequency().index());
    map.putInt("ReferentMode",param.getReferentMode().index());
    map.putArray("ChannelMode",Arguments.fromArray(param.getRawChannelMode()));
    map.putArray("ChannelGain",Arguments.fromArray(param.getRawChannelGain()));
  }

  @Override
  public void setAmplifierParam(ReadableMap param) throws Exception {
    ((NeuroEEG)sensor).setAmplifierParam(new NeuroEEGAmplifierParam(
      param.getBoolean("ReferentResistMesureAllow"),
      (int) param.getInt("Frequency"),
      (int) param.getInt("ReferentMode"),
      ConvertReactToIntArray(param.getArray("ChannelMode")),
      ConvertReactToIntArray(param.getArray("ChannelGain"))
    ));
  }

  @Override
  public int getSamplingFrequencyResist() throws Exception {
    return ((NeuroEEG)sensor).getSamplingFrequencyResist().index();
  }

  private static void ConvertResistChannelsData(ResistChannelsData value, WritableMap result) {
    result.putInt("PackNum", value.getPackNum());
    result.putDouble("A1", value.getA1());
    result.putDouble("A2", value.getA2());
    result.putDouble("Bias", value.getBias());
    result.putArray("Values", Arguments.fromArray(value.getValues()));
  }
}