package com.neurosdk2;

import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.neurosdk2.neuro.types.FPGData;
import com.neurosdk2.neuro.types.MEMSData;
import com.neurosdk2.neuro.types.SignalChannelsData;

import java.util.ArrayList;

public class Utils {

  protected static final String GUID_ID = "GUID";
  protected static final String DATA_ID = "data";

  public static void sendEvent(ReactContext reactContext, String eventName, @Nullable Object params)
  {
    reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
  }

  static void ConvertSignalChannelsData(SignalChannelsData value, WritableMap result) {
    result.putInt("PackNum", value.getPackNum());
    result.putInt("Marker", value.getMarker());
    result.putArray("Samples", Arguments.fromArray(value.getSamples()));
  }

  static void ConvertFPGData(FPGData value, WritableMap result){
    result.putInt("PackNum", value.getPackNum());
    result.putDouble("IrAmplitude", value.getIrAmplitude());
    result.putDouble("RedAmplitude", value.getRedAmplitude());
  }

  static void ConvertMEMSData(MEMSData value, WritableMap result){
    WritableMap accelerometerMap = Arguments.createMap();
    accelerometerMap.putDouble("X", value.getAccelerometer().getX());
    accelerometerMap.putDouble("Y", value.getAccelerometer().getY());
    accelerometerMap.putDouble("Z", value.getAccelerometer().getZ());
    WritableMap gyroscopeMap = Arguments.createMap();
    gyroscopeMap.putDouble("X", value.getGyroscope().getX());
    gyroscopeMap.putDouble("Y", value.getGyroscope().getY());
    gyroscopeMap.putDouble("Z", value.getGyroscope().getZ());
    result.putInt("PackNum", value.getPackNum());
    result.putMap("Accelerometer", accelerometerMap);
    result.putMap("Gyroscope",gyroscopeMap);
  }

  static int[] ConvertReactToIntArray(ReadableArray array)
  {
    int[] result = new int[array.size()];
    ArrayList arrayList = Arguments.toList(array);
    for (int i = 0; i < result.length; i++) {
      result[i] = (int)arrayList.get(i);
    }
    return result;
  }
  static boolean[] ConvertReactToBoolArray(ReadableArray array)
  {
    boolean[] result = new boolean[array.size()];
    ArrayList arrayList = Arguments.toList(array);
    for (int i = 0; i < result.length; i++) {
      result[i] = (boolean)arrayList.get(i);
    }
    return result;
  }
}
