//
//  AFullHeader.h
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import "AFullSensor.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation AFullSensor

- (void) addCallbacks{
    __weak AFullSensor* weakSelf = self;
    [_sensor setConnectionStateCallback:^(NTSensorState state) {
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        NSDictionary *data = @{ @"state" : [NSNumber numberWithUnsignedInt:state]};
        [res setObject:data forKey:DATA_ID];
        @try {
            NSString* eventName = CONNECTION_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:eventName params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
        
    }];
    [_sensor setBatteryCallback:^(NSNumber * _Nonnull power) {
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        NSDictionary *data = @{ @"power" : power};
        [res setObject:data forKey:DATA_ID];
        @try {
            NSString* eventName = BATTERY_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:eventName params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
}
- (void) removeCallbacks{
    [_sensor setConnectionStateCallback:nil];
    [_sensor setBatteryCallback:nil];
}

- (void)closeSensor{
    [self removeCallbacks];
    _sensor = nil;
}

- (void)connectSensor{
    [self.sensor Connect];
}

- (void)disconnectSensor{
    [self.sensor Disconnect];
}

- (void)execCommand:(NSNumber*_Nonnull)command{
    [self.sensor ExecCommand:(NTSensorCommand)[command intValue]];
}

- (NSString *_Nonnull)getAddress{
    return [_sensor Address];
}

- (NSNumber *_Nonnull)getBattPower{
    return [_sensor BattPower];
}

- (NSNumber *_Nonnull)getChannelsCount{
    return [_sensor ChannelsCount];
}

- (NSNumber *_Nonnull)getDataOffset{
    return @([_sensor DataOffset]);
}

- (NSNumber *_Nonnull)getFirmwareMode{
    return @([_sensor FirmwareMode]);
}

- (NSNumber *_Nonnull)getGain{
    return @([_sensor Gain]);
}

- (NSString *_Nonnull)getName{
    return [_sensor Name];
}

- (NSNumber *_Nonnull)getSamplingFrequency{
    return @([_sensor SamplingFrequency]);
}

- (NSNumber *_Nonnull)getSensFamily{
    return @([_sensor SensFamily]);
}

- (NSString *_Nonnull)getSerialNumber{
    return [_sensor SerialNumber];
}

- (NSNumber *_Nonnull)getState{
    return @([_sensor State]);
}

- (NSDictionary *_Nonnull)getVersion{
    NTSensorVersion* version = [_sensor Version];
    NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:
        @(version.FwMajor), @"FwMajor",
        @(version.FwMinor), @"FwMinor",
        @(version.FwPatch), @"FwPatch",
        @(version.HwMajor), @"HwMajor",
        @(version.HwMinor), @"HwMinor",
        @(version.HwPatch), @"HwPatch",
        @(version.ExtMajor), @"ExtMajor",
                                  nil];
    return currentState;
}

- (NSNumber *_Nonnull)isSupportedCommand:(NSNumber*_Nonnull)command{
    BOOL supported = [_sensor IsSupportedCommand:(NTSensorCommand)[command intValue]];
    return [[NSNumber alloc] initWithBool:supported];
}

- (NSNumber *_Nonnull)isSupportedFeature:(NSNumber*_Nonnull)feature{
    BOOL supported = [_sensor IsSupportedFeature:(NTSensorFeature)[feature intValue]];
    return [[NSNumber alloc] initWithBool:supported];
}

- (NSNumber *_Nonnull)isSupportedParameter:(NSNumber*_Nonnull)parameter{
    BOOL supported = [_sensor IsSupportedParameter:(NTSensorParameter)[parameter intValue]];
    return [[NSNumber alloc] initWithBool:supported];
}

- (void)setName:(NSString *_Nonnull)name{
    _sensor.Name = name;
}

- (void)setSerialNumber:(NSString *_Nonnull)sn{
    _sensor.SerialNumber = sn;
}

- (NSArray<NSNumber *> *_Nonnull)supportedCommands{
    return [_sensor Commands];
}

- (NSArray<NSNumber *> *_Nonnull)supportedFeatures{
    return [_sensor Features];
}

- (NSArray<NSDictionary *> *_Nonnull)supportedParameters{
    NSMutableArray* array = [[NSMutableArray alloc] initWithCapacity:_sensor.Parameters.count];
        for (NTParameterInfo *info in _sensor.Parameters) {
            NSDictionary* infoDic = @{
                @"Param" : [NSNumber numberWithInt:info.Param],
                @"ParamAccess" : [NSNumber numberWithInt:info.ParamAccess]};
            [array addObject:infoDic];
        }
    return array;
}

// throwable


- (NSNumber *_Nonnull)getAmpMode{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getSamplingFrequencyResist{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


- (NSNumber *_Nonnull)getSamplingFrequencyFPG{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getIrAmplitude{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getRedAmplitude{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setIrAmplitude:(NTIrAmplitude)amp{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setRedAmplitude:(NTRedAmplitude)amp{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)pingNeuroSmart:(NSNumber*_Nonnull)marker{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


- (NSNumber * _Nonnull)isSupportedFilter:(NSNumber*_Nonnull)filter {
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)getSupportedFilters:(NSMutableArray<NSNumber *> *_Nonnull)result{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)getHardwareFilters:(NSMutableArray<NSNumber *> *_Nonnull)result{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getExtSwInput{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getADCInput{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSDictionary *_Nonnull)getStimulatorMAState{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSDictionary *_Nonnull)getStimulatorParam{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)getMotionAssistantParam:(NSMutableDictionary *_Nonnull)result{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)getMotionCounterParam:(NSMutableDictionary *_Nonnull)result{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getMotionCounter{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getColor{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getMEMSCalibrateState{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getSamplingFrequencyResp{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber * _Nonnull)getSamplingFrequencyEnvelope {
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getSignalType{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber * _Nonnull)getElectrodeState {
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setHardwareFilters:(NSArray *_Nonnull)filters{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setFirmwareMode:(NTSensorFirmwareMode)mode{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setSamplingFrequency:(NTSensorSamplingFrequency)sf{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setGain:(NTSensorGain)gain{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setDataOffset:(NTSensorDataOffset)offset{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setExtSwInput:(NTSensorExternalSwitchInput)extSwInp{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setADCInput:(NTSensorADCInput)adcInp{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setStimulatorParam:(NTCallibriStimulationParams*_Nonnull)param{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setMotionAssistantParam:(NTCallibriMotionAssistantParams*_Nonnull)param{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setMotionCounterParam:(NTCallibriMotionCounterParam*_Nonnull)param{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setSignalType:(NTCallibriSignalType)type{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


- (NSNumber *_Nonnull)getAccSens{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getGyroSens{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSNumber *_Nonnull)getSamplingFrequencyMEMS{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setAccSens:(NTSensorAccelerometerSensitivity)accSens{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setGyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


- (NSNumber *_Nonnull)getSurveyId{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (NSDictionary *_Nonnull)getAmplifierParam{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setSurveyId:(NSNumber *_Nonnull)id{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setAmplifierParam:(NTNeuroEEGAmplifierParam*_Nonnull)param{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


- (NSArray *_Nullable)getSupportedChannels{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}



- (NSDictionary *_Nonnull)getAmplifierParamBB2{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}

- (void)setAmplifierParamBB2:(NTBrainBit2AmplifierParam*_Nonnull)param{
    @throw([NSException exceptionWithName:@"UnsupportedOperationException" reason:@"Unsupported Sensor parameter" userInfo:nil]);
}


@end
