//
//  BrainBitImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "BrainBit2Impl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation BrainBit2Impl

- (instancetype)init:(NSString *)guid
              sensor:(id)bb
           rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = bb;
        self.guid = guid;
        self.rncontext = rncontext;
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    __weak BrainBit2Impl* weakSelf = self;
    [((NTBrainBit2*)(self.sensor)) setSignalDataCallback:^(NSArray<NTSignalChannelsData *> * _Nonnull signals) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTSignalChannelsData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Samples" : signal.Samples
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* csignalEvent = BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

    [((NTBrainBit2*)(self.sensor)) setResistDataCallback:^(NSArray<NTResistRefChannelsData*>*_Nonnull resists) {
        @try {
            NSMutableArray* resistArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTResistRefChannelsData* resist in resists) {
                NSDictionary *resistDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"Samples" : resist.Samples,
                    @"Referents" : resist.Referents
                };
                [resistArray addObject:resistDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:resistArray forKey:DATA_ID];
            NSString* bbresistEvent = BRAINBIT2_RESIST_RECEIVED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:bbresistEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTBrainBit2*)(self.sensor)) setFPGDataCallback:^(NSArray<NTFPGData *> * _Nonnull FPGData) {
        NSMutableArray* FPGDataArray = [[NSMutableArray alloc] initWithCapacity: [FPGData count]];
                for (NTFPGData* data in FPGData) {
                    NSDictionary *dataDic = @{ @"PackNum" : [NSNumber numberWithInt:data.PackNum],
                                               @"IrAmplitude" : data.IrAmplitude,
                                               @"RedAmplitude" : data.RedAmplitude
                    };
                    [FPGDataArray addObject:dataDic];
                }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:FPGDataArray forKey:DATA_ID];
        NSString* fpgEvent = NEUROSMART_FPGDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent: fpgEvent params:res];
    }];
    [((NTBrainBit2*)(self.sensor)) setMEMSDataCallback:^(NSArray<NTMEMSData *> * _Nonnull MEMSData) {
        NSMutableArray* MEMSDataArray = [[NSMutableArray alloc] initWithCapacity: [MEMSData count]];
        for (NTMEMSData* data in MEMSData) {
            NSDictionary* AccelerometerDic = @{
                @"X" : [NSNumber numberWithDouble: data.Accelerometer.X],
                @"Y" : [NSNumber numberWithDouble: data.Accelerometer.Y],
                @"Z" : [NSNumber numberWithDouble: data.Accelerometer.Z]
            };
            NSDictionary* GyroscopeDic = @{
                @"X" : [NSNumber numberWithDouble: data.Gyroscope.X],
                @"Y" : [NSNumber numberWithDouble: data.Gyroscope.Y],
                @"Z" : [NSNumber numberWithDouble: data.Gyroscope.Z]
            };
            NSDictionary *dataDic = @{
                @"Accelerometer" : AccelerometerDic,
                @"Gyroscope" : GyroscopeDic
            };
            [MEMSDataArray addObject:dataDic];
        }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:MEMSDataArray forKey:DATA_ID];
        NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:memsEvent params:res];
    }];
    [((NTBrainBit2*)(self.sensor)) setAmpModeCallback:^(NTSensorAmpMode ampMode) {
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:[NSNumber numberWithUnsignedInt:ampMode] forKey:DATA_ID];
        NSString* ampmodeEvent = AMPMODE_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:ampmodeEvent params:res];
    }];
}
- (void) removeCallbacks{
    [super removeCallbacks];
    [((NTBrainBit2*)(self.sensor)) setSignalDataCallback:nil];
    [((NTBrainBit2*)(self.sensor)) setResistDataCallback:nil];
    [((NTBrainBit2*)(self.sensor)) setFPGDataCallback:nil];
    [((NTBrainBit2*)(self.sensor)) setMEMSDataCallback:nil];
    [((NTBrainBit2*)(self.sensor)) setAmpModeCallback:nil];
}

- (NSNumber *)getSamplingFrequencyMEMS{
    return @([(NTBrainBit2*)(self.sensor) SamplingFrequencyMEMS]);
}
- (NSNumber *)getSamplingFrequencyFPG{
    return @([(NTBrainBit2*)(self.sensor) SamplingFrequencyFPG]);
}

- (NSNumber *)getSamplingFrequencyResist{
    return @([(NTBrainBit2*)(self.sensor) SamplingFrequencyResist]);
}
- (NSNumber *)getIrAmplitude{
    return @([(NTBrainBit2*)(self.sensor) IrAmplitude]);
}
- (void)setIrAmplitude:(NTIrAmplitude)amp{
    ((NTBrainBit2*)(self.sensor)).IrAmplitude = amp;
}
- (NSNumber *)getRedAmplitude{
    return @([(NTBrainBit2*)(self.sensor) RedAmplitude]);
}
- (void)setRedAmplitude:(NTRedAmplitude)amp{
    ((NTBrainBit2*)(self.sensor)).RedAmplitude = amp;
}
- (NSNumber *)getAmpMode{
    return @([(NTBrainBit2*)(self.sensor) AmpMode]);
}
- (void)setAccSens:(NTSensorAccelerometerSensitivity)accSens{
    ((NTBrainBit2*)self.sensor).AccSens = accSens;
}
- (NSNumber *_Nonnull)getAccSens{
    return @([((NTBrainBit2*)self.sensor) AccSens]);
}
- (void)setGyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    ((NTBrainBit2*)self.sensor).GyroSens = gyroSens;
}
- (NSNumber *_Nonnull)getGyroSens{
    return @([((NTBrainBit2*)self.sensor) GyroSens]);
}
- (NSArray *_Nullable)getSupportedChannels{
    NSArray<NTEEGChannelInfo*>* channels = [(NTBrainBit2*)self.sensor SupportedChannels];
    NSMutableArray* channelsArray = [[NSMutableArray alloc] initWithCapacity: [channels count]];
    for (NTEEGChannelInfo* channel in channels) {
        NSDictionary *dataDic = @{
            @"Id" : [NSNumber numberWithInt:channel.Id],
            @"ChType" : [NSNumber numberWithInt:channel.ChType],
            @"Name" : channel.Name,
            @"Num" : [NSNumber numberWithUnsignedShort:channel.Num],
        };
        [channelsArray addObject:dataDic];
    }
    return [channelsArray copy];
}
- (NSDictionary *_Nonnull)getAmplifierParamBB2{
    NTBrainBit2AmplifierParam* param = [(NTBrainBit2*)self.sensor AmplifierParam];
    NSDictionary* result = @{
        @"Current": @(param.Current),
        @"ChSignalMode": param.ChSignalMode,
        @"ChResistUse": param.ChResistUse,
        @"ChGain":param.ChGain
    };
    return result;
}
- (void)setAmplifierParamBB2:(NTBrainBit2AmplifierParam*_Nonnull)param{
    ((NTBrainBit2*)self.sensor).AmplifierParam = param;
}
- (void)pingNeuroSmart:(NSNumber *)marker{
    [(NTBrainBit2*)(self.sensor) PingNeuroSmart: [marker charValue]];
}

@end
