//
//  BrainBitBlackImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "BrainBitBlackImpl.h"
#import "Constants.h"
#import "AFullSensorProtected.h"

@implementation BrainBitBlackImpl

- (instancetype)init:(NSString *)guid
              sensor:(id)bb
           rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = bb;
        self.guid = guid;
        self.rncontext = rncontext;
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    __weak BrainBitBlackImpl* weakSelf = self;
    [((NTBrainBitBlack*)(self.sensor)) setFPGDataCallbackNeuroSmart:^(NSArray<NTFPGData *> * _Nonnull FPGData) {
        NSMutableArray* FPGDataArray = [[NSMutableArray alloc] initWithCapacity: [FPGData count]];
                for (NTFPGData* data in FPGData) {
                    NSDictionary *dataDic = @{ @"PackNum" : [NSNumber numberWithInt:data.PackNum],
                                               @"IrAmplitude" : data.IrAmplitude,
                                               @"RedAmplitude" : data.RedAmplitude
                    };
                    [FPGDataArray addObject:dataDic];
                }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:FPGDataArray forKey:DATA_ID];
        NSString* fpgEvent = NEUROSMART_FPGDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent: fpgEvent params:res];
    }];
    [((NTBrainBitBlack*)(self.sensor)) setMEMSDataCallback:^(NSArray<NTMEMSData *> * _Nonnull MEMSData) {
        NSMutableArray* MEMSDataArray = [[NSMutableArray alloc] initWithCapacity: [MEMSData count]];
        for (NTMEMSData* data in MEMSData) {
            NSDictionary* AccelerometerDic = @{
                @"X" : [NSNumber numberWithDouble: data.Accelerometer.X],
                @"Y" : [NSNumber numberWithDouble: data.Accelerometer.Y],
                @"Z" : [NSNumber numberWithDouble: data.Accelerometer.Z]
            };
            NSDictionary* GyroscopeDic = @{
                @"X" : [NSNumber numberWithDouble: data.Gyroscope.X],
                @"Y" : [NSNumber numberWithDouble: data.Gyroscope.Y],
                @"Z" : [NSNumber numberWithDouble: data.Gyroscope.Z]
            };
            NSDictionary *dataDic = @{
                @"Accelerometer" : AccelerometerDic,
                @"Gyroscope" : GyroscopeDic
            };
            [MEMSDataArray addObject:dataDic];
        }
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:MEMSDataArray forKey:DATA_ID];
        NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:memsEvent params:res];
    }];
    [((NTBrainBitBlack*)(self.sensor)) setAmpModeCallback:^(NTSensorAmpMode ampMode) {
        NSMutableDictionary* res = [NSMutableDictionary new];
        [res setObject:weakSelf.guid forKey:GUID_ID];
        [res setObject:[NSNumber numberWithUnsignedInt:ampMode] forKey:DATA_ID];
        NSString* ampmodeEvent = AMPMODE_CHANGED_EVENT_NAME;
        [weakSelf.rncontext neuroSendEvent:ampmodeEvent params:res];
    }];
}
- (void) removeCallbacks{
    [super removeCallbacks];
    [((NTBrainBitBlack*)(self.sensor)) setFPGDataCallbackNeuroSmart:nil];
    [((NTBrainBitBlack*)(self.sensor)) setMEMSDataCallback:nil];
    [((NTBrainBitBlack*)(self.sensor)) setAmpModeCallback:nil];
}

- (NSNumber *)getSamplingFrequencyMEMS{
    return @([(NTBrainBitBlack*)(self.sensor) SamplingFrequencyMEMS]);
}
- (NSNumber *)getSamplingFrequencyFPG{
    return @([(NTBrainBitBlack*)(self.sensor) SamplingFrequencyFPG]);
}

- (NSNumber *)getSamplingFrequencyResist{
    return @([(NTBrainBitBlack*)(self.sensor) SamplingFrequencyResist]);
}
- (void)setIrAmplitude:(NTIrAmplitude)amp{
    ((NTBrainBitBlack*)(self.sensor)).IrAmplitudeHeadband = amp;
}
- (NSNumber *)getRedAmplitude{
    return @([(NTBrainBitBlack*)(self.sensor) RedAmplitudeHeadband]);
}
- (void)setRedAmplitude:(NTRedAmplitude)amp{
    ((NTBrainBitBlack*)(self.sensor)).RedAmplitudeHeadband = amp;
}
- (NSNumber *)getAmpMode{
    return @([(NTBrainBitBlack*)(self.sensor) AmpMode]);
}
- (void)setAccSens:(NTSensorAccelerometerSensitivity)accSens{
    ((NTBrainBitBlack*)self.sensor).AccSens = accSens;
}
- (NSNumber *_Nonnull)getAccSens{
    return @([((NTBrainBitBlack*)self.sensor) AccSens]);
}
- (void)setGyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    ((NTBrainBitBlack*)self.sensor).GyroSens = gyroSens;
}
- (NSNumber *_Nonnull)getGyroSens{
    return @([((NTBrainBitBlack*)self.sensor) GyroSens]);
}
- (void)pingNeuroSmart:(NSNumber *)marker{
    [(NTBrainBitBlack*)(self.sensor) PingNeuroSmart: [marker charValue]];
}


@end
