//
//  CallibriImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "CallibriImpl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation CallibriImpl

- (instancetype _Nonnull )init:(NSString*_Nonnull)guid
                        sensor:(NTSensor*_Nonnull)sensor
                     rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = sensor;
        self.guid = guid;
        self.rncontext = rncontext;
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    
    __weak CallibriImpl* weakSelf = self;
    [((NTCallibri*)self.sensor) setSignalCallback:^(NSArray<NTCallibriSignalData *> * _Nonnull signals) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
                    for (NTCallibriSignalData* signal in signals) {
                        NSDictionary *signalDic = @{
                            @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                            @"Samples" : signal.Samples
                        };
                        [signalsArray addObject:signalDic];
                    }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* csignalEvent = CALLIBRI_SIGNAL_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTCallibri*)self.sensor) setElectrodeStateCallback:^(NTCallibriElectrodeState callibriElectrodeState) {
        @try {
            NSDictionary *state = @{
                @"CallibriElectrodeState" : [NSNumber numberWithInt:callibriElectrodeState]
            };
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:state forKey:DATA_ID];
            NSString* electrodeEvent = CALLIBRI_ELECTRODE_STATE_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:electrodeEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
        
    }];
    [((NTCallibri*)self.sensor) setEnvelopeDataCallback:^(NSArray<NTCallibriEnvelopeData *> * _Nonnull envelopes) {
        @try {
            NSMutableArray* envelopesArray = [[NSMutableArray alloc] initWithCapacity: [envelopes count]];
                    for (NTCallibriEnvelopeData* envelope in envelopes) {
                        NSDictionary *envelopeDic = @{
                            @"PackNum" : [NSNumber numberWithInt:envelope.PackNum],
                            @"Sample" : envelope.Sample
                        };
                        [envelopesArray addObject:envelopeDic];
                    }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:envelopesArray forKey:DATA_ID];
            NSString* envelopeEvent = CALLIBRI_ENVELOPE_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:envelopeEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
        
    }];
    [((NTCallibri*)self.sensor) setRespirationCallback:^(NSArray<NTCallibriRespirationData *> * _Nonnull callibriRespirationData) {
        @try {
            NSMutableArray* callibriRespirationDataArray = [[NSMutableArray alloc] initWithCapacity: [callibriRespirationData count]];
                    for (NTCallibriRespirationData* respData in callibriRespirationData) {
                        NSDictionary *callibriRespirationDataDic = @{
                            @"PackNum" : [NSNumber numberWithInt:respData.PackNum],
                            @"Samples" : respData.Samples
                        };
                        [callibriRespirationDataArray addObject:callibriRespirationDataDic];
                    }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:callibriRespirationDataArray forKey:DATA_ID];
            NSString* respirationEvent = CALLIBRI_RESPIRATION_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:respirationEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
        
    }];
    [((NTCallibri*)self.sensor) setQuaternionDataCallback:^(NSArray<NTQuaternionData *> * _Nonnull quaternionData) {
        @try {
            NSMutableArray* quaternionDataArray = [[NSMutableArray alloc] initWithCapacity: [quaternionData count]];
                    for (NTQuaternionData* data in quaternionData) {
                        NSDictionary *dataDic = @{
                            @"PackNum" : [NSNumber numberWithUnsignedInt:data.PackNum],
                            @"W" : [NSNumber numberWithFloat: data.W],
                            @"X" : [NSNumber numberWithFloat: data.X],
                            @"Y" : [NSNumber numberWithFloat: data.Y],
                            @"Z" : [NSNumber numberWithFloat: data.Z]
                        };
                        [quaternionDataArray addObject:dataDic];
                    }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:quaternionDataArray forKey:DATA_ID];
            NSString* quaterEvent = QUATERNIONDATA_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:quaterEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
        
    }];
    [((NTCallibri*)self.sensor) setMEMSDataCallback:^(NSArray<NTMEMSData *> * _Nonnull MEMSData) {
        @try {
            NSMutableArray* MEMSDataArray = [[NSMutableArray alloc] initWithCapacity: [MEMSData count]];
                    for (NTMEMSData* data in MEMSData) {
                        NSDictionary* AccelerometerDic = @{
                            @"X" : [NSNumber numberWithDouble: data.Accelerometer.X],
                            @"Y" : [NSNumber numberWithDouble: data.Accelerometer.Y],
                            @"Z" : [NSNumber numberWithDouble: data.Accelerometer.Z]
                        };
                        NSDictionary* GyroscopeDic = @{
                            @"X" : [NSNumber numberWithDouble: data.Gyroscope.X],
                            @"Y" : [NSNumber numberWithDouble: data.Gyroscope.Y],
                            @"Z" : [NSNumber numberWithDouble: data.Gyroscope.Z]
                        };
                        NSDictionary *dataDic = @{
                            @"Accelerometer" : AccelerometerDic,
                            @"Gyroscope" : GyroscopeDic
                        };
                        [MEMSDataArray addObject:dataDic];
                    }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:MEMSDataArray forKey:DATA_ID];
            NSString* memsEvent = MEMSDATA_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:memsEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
}

- (void)removeCallbacks{
    [super removeCallbacks];
    [((NTCallibri*)self.sensor) setSignalCallback:nil];
    [((NTCallibri*)self.sensor) setElectrodeStateCallback:nil];
    [((NTCallibri*)self.sensor) setEnvelopeDataCallback:nil];
    [((NTCallibri*)self.sensor) setRespirationCallback:nil];
    [((NTCallibri*)self.sensor) setQuaternionDataCallback:nil];
    [((NTCallibri*)self.sensor) setMEMSDataCallback:nil];
}

- (NSNumber *_Nonnull)getADCInput{
    return @([((NTCallibri*)self.sensor) ADCInput]);
}

- (NSNumber *_Nonnull)getAccSens{
    return @([((NTCallibri*)self.sensor) AccSens]);
}

- (NSNumber *_Nonnull)getColor{
    return @([((NTCallibri*)self.sensor) Color]);
}

- (NSNumber *_Nonnull)getExtSwInput{
    return @([((NTCallibri*)self.sensor) ExtSwInput]);
}

- (NSNumber *_Nonnull)getGyroSens{
    return @([((NTCallibri*)self.sensor) GyroSens]);
}

- (NSNumber *_Nonnull)isSupportedFilter:(NSNumber*_Nonnull)filter{
    BOOL supported = [((NTCallibri*)self.sensor) isSupportedFilter:(NTSensorFilter)[filter intValue]];
    return [[NSNumber alloc] initWithBool:supported];
}

- (void)getSupportedFilters:(NSMutableArray<NSNumber *> *_Nonnull)result{
    [result addObjectsFromArray:[((NTCallibri*)self.sensor) SupportedFilters]];
}

- (void)getHardwareFilters:(NSMutableArray<NSNumber *> *_Nonnull)result{
    [result addObjectsFromArray:[((NTCallibri*)self.sensor) HardwareFilters]];
}

- (NSNumber *_Nonnull)getMEMSCalibrateState{
    return @([((NTCallibri*)self.sensor) MEMSCalibrateState]);
}

- (void)getMotionAssistantParam:(NSMutableDictionary *_Nonnull)result{
    NTCallibriMotionAssistantParams* params = [(NTCallibri*)self.sensor MotionAssistantParam];
    [result setObject:@(params.GyroStart) forKey:@"GyroStart"];
    [result setObject:@(params.GyroStop) forKey:@"GyroStop"];
    [result setObject:@(params.Limb) forKey:@"Limb"];
    [result setObject:@(params.MinPauseMs) forKey:@"MinPauseMs"];
}

- (NSNumber *_Nonnull)getMotionCounter{
    return @([((NTCallibri*)self.sensor) MotionCounter]);
}

- (void)getMotionCounterCallibri:(NSMutableDictionary *_Nonnull)result{
    NTCallibriMotionCounterParam* params = [(NTCallibri*)self.sensor MotionCounterParam];
    [result setObject:@(params.InsenseThresholdMG) forKey:@"InsenseThresholdMG"];
    [result setObject:@(params.InsenseThresholdSample) forKey:@"InsenseThresholdSample"];
}

- (NSNumber *_Nonnull)getSamplingFrequencyMEMS{
    return @([((NTCallibri*)self.sensor) SamplingFrequencyMEMS]);
}

- (NSNumber *_Nonnull)getSamplingFrequencyResp{
    return @([((NTCallibri*)self.sensor) SamplingFrequencyResp]);
}

- (NSNumber *_Nonnull)getSamplingFrequencyEnvelope{
    return @([((NTCallibri*)self.sensor) SamplingFrequencyEnvelope]);
}

- (NSNumber *_Nonnull)getSignalType{
    return @([((NTCallibri*)self.sensor) SignalType]);
}

- (NSNumber *_Nonnull)getElectrodeState{
    return @([((NTCallibri*)self.sensor) ElectrodeState]);
}

- (NSDictionary *_Nonnull)getStimulatorMAState{
    NTCallibriStimulatorMAState* state = [(NTCallibri*)self.sensor StimulatorMAState];
        NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:
                                       [NSNumber numberWithUnsignedInteger:state.stimulatorState], @"StimulatorState",
                                       [NSNumber numberWithUnsignedInteger:state.MAState], @"MAState", nil];
    return currentState;
}

- (NSDictionary *_Nonnull)getStimulatorParam{
    NTCallibriStimulationParams* param = [(NTCallibri*)self.sensor StimulatorParam];
        NSDictionary* currentState = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithUnsignedInteger:param.Current], @"Current",
        [NSNumber numberWithUnsignedInteger:param.PulseWidth], @"PulseWidth",
        [NSNumber numberWithUnsignedInteger:param.Frequency], @"Frequency",
        [NSNumber numberWithUnsignedInteger:param.StimulusDuration], @"StimulusDuration",
        nil];
    return currentState;
}

- (void)setADCInput:(NTSensorADCInput)adcInp{
    ((NTCallibri*)self.sensor).ADCInput = adcInp;
}

- (void)setAccSens:(NTSensorAccelerometerSensitivity)accSens{
    ((NTCallibri*)self.sensor).AccSens = accSens;
}

- (void)setDataOffset:(NTSensorDataOffset)offset{
    ((NTCallibri*)self.sensor).DataOffset = offset;
}

- (void)setExtSwInput:(NTSensorExternalSwitchInput)extSwInp{
    ((NTCallibri*)self.sensor).ExtSwInput = extSwInp;
}

- (void)setFirmwareMode:(NTSensorFirmwareMode)mode{
    ((NTCallibri*)self.sensor).FirmwareMode = mode;
}

- (void)setGain:(NTSensorGain)gain{
    ((NTCallibri*)self.sensor).Gain = gain;
}

- (void)setGyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    ((NTCallibri*)self.sensor).GyroSens = gyroSens;
}

- (void)setHardwareFilters:(NSArray *_Nonnull)filters{
    ((NTCallibri*)self.sensor).HardwareFilters = filters;
}

- (void)setMotionAssistantParam:(NTCallibriMotionAssistantParams*_Nonnull)param{
    ((NTCallibri*)self.sensor).MotionAssistantParam = param;
}

- (void)setMotionCounterParam:(NTCallibriMotionCounterParam*_Nonnull)param{
    ((NTCallibri*)self.sensor).MotionCounterParam = param;
}

- (void)setSamplingFrequency:(NTSensorSamplingFrequency)sf{
    ((NTCallibri*)self.sensor).SamplingFrequency = sf;
}

- (void)setSignalType:(NTCallibriSignalType)type{
    ((NTCallibri*)self.sensor).SignalType = type;
}

- (void)setStimulatorParam:(NTCallibriStimulationParams*_Nonnull)param{
    ((NTCallibri*)self.sensor).StimulatorParam = param;
}

@end
