//
//  NeuroEEGImpl.m
//  Neurosdk2
//
//  Created by Aseatari on 25.10.2023.
//

#import <Foundation/Foundation.h>
#import "NeuroEEGImpl.h"
#import "AFullSensorProtected.h"
#import "Constants.h"

@implementation NeuroEEGImpl

- (instancetype _Nonnull )init:(NSString*_Nonnull)guid
                        sensor:(NTSensor*_Nonnull)sensor
                     rncontext:(Neurosdk2* _Nonnull)rncontext{
    self = [super init];
    if (self) {
        self.sensor = sensor;
        self.guid = guid;
        self.rncontext = rncontext;
        NTNeuroEEGFSStatus* fsStatus = [(NTNeuroEEG*)self.sensor FSStatus];
        if (fsStatus.Status == NTSensorFSStatusOK) {
            [(NTNeuroEEG*)self.sensor ExecCommand: NTSensorCommandFileSystemStreamClose];
            [(NTNeuroEEG*)self.sensor ExecCommand: NTSensorCommandFileSystemDisable];
        }
        [self addCallbacks];
    }
    return self;
}

- (void) addCallbacks{
    [super addCallbacks];
    
    __weak NeuroEEGImpl* weakSelf = self;
    [((NTNeuroEEG*)self.sensor) setAmpModeCallback:^(NTSensorAmpMode ampMode) {
        @try {
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:[NSNumber numberWithInt:ampMode] forKey:DATA_ID];
            NSString* ampModeEvent = AMPMODE_CHANGED_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:ampModeEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setSignalCallback:^(NSArray<NTSignalChannelsData *> * _Nonnull signals) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTSignalChannelsData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Samples" : signal.Samples
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:signalsArray forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_SIGNAL_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setResistCallback:^(NSArray<NTResistChannelsData *> * _Nonnull resists) {
        @try {
            NSMutableArray* resistArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTResistChannelsData* resist in resists) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"A1" : [NSNumber numberWithDouble:resist.A1.intValue],
                    @"A2" : [NSNumber numberWithDouble:resist.A2.intValue],
                    @"Bias" : [NSNumber numberWithDouble:resist.Bias.intValue],
                    @"Values" : resist.Values
                };
                [resistArray addObject:signalDic];
            }
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:resistArray forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_RESIST_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];
    [((NTNeuroEEG*)self.sensor) setSignalResistCallback:^(NSArray<NTSignalChannelsData *> * _Nonnull signals, NSArray<NTResistChannelsData *> * _Nonnull resists) {
        @try {
            NSMutableArray* signalsArray = [[NSMutableArray alloc] initWithCapacity: [signals count]];
            for (NTSignalChannelsData* signal in signals) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:signal.PackNum],
                    @"Marker" : [NSNumber numberWithInt:signal.Marker],
                    @"Samples" : signal.Samples
                };
                [signalsArray addObject:signalDic];
            }
            NSMutableArray* resistsArray = [[NSMutableArray alloc] initWithCapacity: [resists count]];
            for (NTResistChannelsData* resist in resists) {
                NSDictionary *signalDic = @{
                    @"PackNum" : [NSNumber numberWithInt:resist.PackNum],
                    @"A1" : [NSNumber numberWithDouble:resist.A1.intValue],
                    @"A2" : [NSNumber numberWithDouble:resist.A2.intValue],
                    @"Bias" : [NSNumber numberWithDouble:resist.Bias.intValue],
                    @"Values" : resist.Values
                };
                [resistsArray addObject:signalDic];
            }
            NSDictionary *data = @{
                @"signalData" : signalsArray,
                @"resistData" : resistsArray
            };
            NSMutableDictionary* res = [NSMutableDictionary new];
            [res setObject:weakSelf.guid forKey:GUID_ID];
            [res setObject:data forKey:DATA_ID];
            NSString* csignalEvent = NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME;
            [weakSelf.rncontext neuroSendEvent:csignalEvent params:res];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception.description);
        }
    }];

}

- (void)removeCallbacks{
    [super removeCallbacks];
    [((NTNeuroEEG*)self.sensor) setAmpModeCallback:nil];
    [((NTNeuroEEG*)self.sensor) setSignalCallback:nil];
    [((NTNeuroEEG*)self.sensor) setResistCallback:nil];
    [((NTNeuroEEG*)self.sensor) setSignalResistCallback:nil];

}

- (NSNumber *_Nonnull)getSurveyId{
    return [((NTNeuroEEG*)self.sensor) SurveyId];
}

- (void)setSurveyId:(NSNumber*_Nonnull)id{
    ((NTNeuroEEG*)self.sensor).SurveyId = id;
}

- (NSNumber *_Nonnull)getAmpMode{
    return @([((NTNeuroEEG*)self.sensor) AmpMode]);
}

- (NSArray *_Nullable)getSupportedChannels{
    NSArray<NTEEGChannelInfo*>* channels = [(NTNeuroEEG*)self.sensor SupportedChannels];
    NSMutableArray* channelsArray = [[NSMutableArray alloc] initWithCapacity: [channels count]];
    for (NTEEGChannelInfo* channel in channels) {
        NSDictionary *dataDic = @{
            @"Id" : [NSNumber numberWithInt:channel.Id],
            @"ChType" : [NSNumber numberWithInt:channel.ChType],
            @"Name" : channel.Name,
            @"Num" : [NSNumber numberWithUnsignedShort:channel.Num],
        };
        [channelsArray addObject:dataDic];
    }
    return [channelsArray copy];
}

- (NSDictionary *_Nonnull)getAmplifierParam{
    NTNeuroEEGAmplifierParam* param = [(NTNeuroEEG*)self.sensor AmplifierParam];
    NSDictionary* result = @{
        @"ReferentResistMesureAllow": @(param.ReferentResistMesureAllow),
        @"Frequency": [NSNumber numberWithInt:param.Frequency],
        @"ReferentMode": [NSNumber numberWithInt:param.ReferentMode],
        @"ChannelMode": param.ChannelMode,
        @"ChannelGain": param.ChannelGain
    };
    return result;
}

- (void)setAmplifierParam:(NTNeuroEEGAmplifierParam*_Nonnull)param{
    ((NTNeuroEEG*)self.sensor).AmplifierParam = param;
}

- (NSNumber *_Nonnull)getSamplingFrequencyResist{
    return @([((NTNeuroEEG*)self.sensor) SamplingFrequencyResist]);
}

@end
