//
//  SensorHolder.h
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#ifndef SensorHolder_h
#define SensorHolder_h

#import <React/RCTBridgeModule.h>
#import <neurosdk2/neurosdk.h>
#import "AFullSensor.h"

#import "CallibriImpl.h"


#import "BrainBitImpl.h"


#import "BrainBitBlackImpl.h"


#import "BrainBit2Impl.h"



#import "NeuroEEGImpl.h"


@interface SensorHolder : NSObject

- (nonnull instancetype)init;

-(void) addSensor:(NSString*_Nonnull)guid
           sensor:(NTSensor*_Nonnull)sensor
        rncontext:(NSObject*_Nonnull)rncontext;

- (void)closeSensor:(NSString *_Nonnull)guid;

- (void)connectSensor:(NSString *_Nonnull)guid
              resolve:(RCTPromiseResolveBlock _Nonnull )resolve
               reject:(RCTPromiseRejectBlock _Nonnull )reject;

- (void)disconnectSensor:(NSString *_Nonnull)guid
                 resolve:(RCTPromiseResolveBlock _Nonnull )resolve
                  reject:(RCTPromiseRejectBlock _Nonnull )reject;

- (void)execCommand:(NSString *_Nonnull)guid
            command:(NSNumber*_Nonnull)command
            resolve:(RCTPromiseResolveBlock _Nonnull )resolve
             reject:(RCTPromiseRejectBlock _Nonnull )reject;

- (NSString *_Nonnull)getAddress:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getBattPower:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getChannelsCount:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getDataOffset:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getFirmwareMode:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getGain:(NSString *_Nonnull)guid;

- (NSString *_Nonnull)getName:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSamplingFrequency:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSensFamily:(NSString *_Nonnull)guid;

- (NSString *_Nonnull)getSerialNumber:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getState:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getVersion:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)isSupportedCommand:(NSString *_Nonnull)guid
                         command:(NSNumber*_Nonnull)command;

- (NSNumber *_Nonnull)isSupportedFeature:(NSString *_Nonnull)guid
                         feature:(NSNumber*_Nonnull)feature;

- (NSNumber *_Nonnull)isSupportedParameter:(NSString *_Nonnull)guid
                         parameter:(NSNumber*_Nonnull)parameter;

- (void)setName:(NSString *_Nonnull)guid
           name:(NSString *_Nonnull)name;

- (void)setSerialNumber:(NSString *_Nonnull)guid
                     sn:(NSString *_Nonnull)sn;

- (NSArray<NSNumber *> *_Nonnull)supportedCommands:(NSString *_Nonnull)guid;

- (NSArray<NSNumber *> *_Nonnull)supportedFeatures:(NSString *_Nonnull)guid;

- (NSArray<NSDictionary *> *_Nonnull)supportedParameters:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getAmpMode:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSamplingFrequencyResist:(NSString *_Nonnull)guid;


- (NSNumber *_Nonnull)getSamplingFrequencyFPG:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getIrAmplitude:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getRedAmplitude:(NSString *_Nonnull)guid;

- (void)setIrAmplitude:(NSString *_Nonnull)guid
                   amp:(NTIrAmplitude)amp;

- (void)setRedAmplitude:(NSString *_Nonnull)guid
                    amp:(NTRedAmplitude)amp;

- (void)pingNeuroSmart:(NSString *_Nonnull)guid
                marker:(NSNumber*_Nonnull)marker
               resolve:(RCTPromiseResolveBlock _Nonnull )resolve
                reject:(RCTPromiseRejectBlock _Nonnull )reject;


- (NSNumber *_Nonnull)isSupportedFilter:(NSString *_Nonnull)guid filter:(NSNumber*_Nonnull)filter;

- (NSArray<NSNumber *> *_Nonnull)getSupportedFilters:(NSString *_Nonnull)guid;

- (NSArray<NSNumber *> *_Nonnull)getHardwareFilters:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getExtSwInput:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getADCInput:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getStimulatorMAState:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getStimulatorParam:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getMotionAssistantParam:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getMotionCounterParam:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getMotionCounter:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getColor:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getMEMSCalibrateState:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSamplingFrequencyResp:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSamplingFrequencyEnvelope:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSignalType:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getElectrodeState:(NSString *_Nonnull)guid;

- (void)setHardwareFilters:(NSString *_Nonnull)guid
                   filters:(NSArray *_Nonnull)filters;

- (void)setFirmwareMode:(NSString *_Nonnull)guid
                   mode:(NTSensorFirmwareMode)mode;

- (void)setSamplingFrequency:(NSString *_Nonnull)guid
                          sf:(NTSensorSamplingFrequency)sf;

- (void)setGain:(NSString *_Nonnull)guid
           gain:(NTSensorGain)gain;

- (void)setDataOffset:(NSString *_Nonnull)guid
               offset:(NTSensorDataOffset)offset;

- (void)setExtSwInput:(NSString *_Nonnull)guid
             extSwInp:(NTSensorExternalSwitchInput)extSwInp;

- (void)setADCInput:(NSString *_Nonnull)guid
             adcInp:(NTSensorADCInput)adcInp;

- (void)setStimulatorParam:(NSString *_Nonnull)guid
                             param:(NTCallibriStimulationParams*_Nonnull)param;

- (void)setMotionAssistantParam:(NSString *_Nonnull)guid
                                  param:(NTCallibriMotionAssistantParams*_Nonnull)param;

- (void)setMotionCounterParam:(NSString *_Nonnull)guid
                                param:(NTCallibriMotionCounterParam*_Nonnull)param;

- (void)setSignalType:(NSString *_Nonnull)guid
                         type:(NTCallibriSignalType)type;


- (NSNumber *_Nonnull)getAccSens:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getGyroSens:(NSString *_Nonnull)guid;

- (NSNumber *_Nonnull)getSamplingFrequencyMEMS:(NSString *_Nonnull)guid;

- (void)setAccSens:(NSString *_Nonnull)guid
           accSens:(NTSensorAccelerometerSensitivity)accSens;

- (void)setGyroSens:(NSString *_Nonnull)guid
           gyroSens:(NTSensorGyroscopeSensitivity)gyroSens;


- (NSNumber *_Nonnull)getSurveyId:(NSString *_Nonnull)guid;

- (NSDictionary *_Nonnull)getAmplifierParam:(NSString *_Nonnull)guid;

- (void)setSurveyId:(NSString *_Nonnull)guid
                             id:(NSNumber *_Nonnull)id;

- (void)setAmplifierParam:(NSString *_Nonnull)guid
                             param:(NTNeuroEEGAmplifierParam *_Nonnull)param;


- (NSArray *_Nullable)getSupportedChannels:(NSString *_Nonnull)guid;



- (NSDictionary *_Nonnull)getAmplifierParamBB2:(NSString *_Nonnull)guid;

- (void)setAmplifierParamBB2:(NSString *_Nonnull)guid
                             param:(NTBrainBit2AmplifierParam *_Nonnull)param;


@end

#endif /* SensorHolder_h */
