//
//  SensorHolder.m
//  Neurosdk2
//
//  Created by Aseatari on 04.05.2023.
//

#import <Foundation/Foundation.h>
#import "SensorHolder.h"
#import "Neurosdk2.h"

@implementation SensorHolder{
    NSMutableDictionary<NSString*, AFullSensor*>* _sensors;
}

- (nonnull instancetype)init{
    self = [super init];
    if (self) {
        _sensors = [NSMutableDictionary new];
      }
    
    return self;
}

-(void) addSensor:(NSString*_Nonnull)guid
           sensor:(NTSensor*_Nonnull)sensor
        rncontext:(Neurosdk2*_Nonnull)rncontext{
    NTSensorFamily family = [sensor SensFamily];
    switch(family){
        case NTSensorFamilyLECallibri:
        case NTSensorFamilyLEKolibri:
            [_sensors setObject:[[CallibriImpl alloc] init:guid
                                                    sensor:sensor
                                                 rncontext:rncontext] forKey:guid];
            break;
        case NTSensorFamilyLEBrainBit:
            [_sensors setObject:[[BrainBitImpl alloc] init:guid
                                                    sensor:sensor
                                                 rncontext:rncontext] forKey:guid];
            break;
        case NTSensorFamilyLEBrainBitBlack:
            [_sensors setObject:[[BrainBitBlackImpl alloc] init:guid
                                                         sensor:sensor
                                                      rncontext:rncontext] forKey:guid];
            break;


        case NTSensorFamilyLEBrainBit2:
        case NTSensorFamilyLEBrainBitFlex:
        case NTSensorFamilyLEBrainBitPro:
            [_sensors setObject:[[BrainBit2Impl alloc] init:guid
                                                     sensor:sensor
                                                  rncontext:rncontext] forKey:guid];
            break;


        case NTSensorFamilyLENeuroEEG:
            [_sensors setObject:[[NeuroEEGImpl alloc] init:guid
                                                         sensor:sensor
                                                      rncontext:rncontext] forKey:guid];
            break;

        default:
            break;
    }
}

- (void)closeSensor:(NSString *_Nonnull)guid{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor closeSensor];
        [_sensors removeObjectForKey:guid];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)connectSensor:(NSString *_Nonnull)guid
              resolve:(RCTPromiseResolveBlock _Nonnull )resolve
               reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor connectSensor];
        resolve(@"success");
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
        reject(@"connectSensor", exception.description, nil);
    }
}

- (void)disconnectSensor:(NSString *_Nonnull)guid
                 resolve:(RCTPromiseResolveBlock _Nonnull )resolve
                  reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor disconnectSensor];
        resolve(@"success");
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
        reject(@"disconnectSensor", exception.description, nil);
    }
}

- (void)execCommand:(NSString *_Nonnull)guid
            command:(NSNumber*_Nonnull)command
            resolve:(RCTPromiseResolveBlock _Nonnull )resolve
             reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor execCommand:command];
        resolve(@"success");
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
        reject(@"execCommand", exception.description, nil);
    }
}

- (NSString *_Nonnull)getAddress:(NSString *_Nonnull)guid{
    NSString* result = @"";
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getAddress];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getBattPower:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getBattPower];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getChannelsCount:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getChannelsCount];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getDataOffset:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getDataOffset];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getFirmwareMode:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getFirmwareMode];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getGain:(NSString *_Nonnull)guid{
    NSNumber* result = @11;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getGain];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSString *_Nonnull)getName:(NSString *_Nonnull)guid{
    NSString* result = @"";
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getName];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getSamplingFrequency:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequency];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getSensFamily:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSensFamily];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSString *_Nonnull)getSerialNumber:(NSString *_Nonnull)guid{
    NSString* result = @"";
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSerialNumber];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getState:(NSString *_Nonnull)guid{
    NSNumber* result = @1;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getState];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getVersion:(NSString *_Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result setDictionary:[sensor getVersion]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)isSupportedCommand:(NSString *_Nonnull)guid
                                 command:(NSNumber*_Nonnull)command{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor isSupportedCommand:command];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)isSupportedFeature:(NSString *_Nonnull)guid
                         feature:(NSNumber*_Nonnull)feature{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor isSupportedFeature:feature];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)isSupportedParameter:(NSString *_Nonnull)guid
                         parameter:(NSNumber*_Nonnull)parameter{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor isSupportedParameter:parameter];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (void)setName:(NSString *_Nonnull)guid
           name:(NSString *_Nonnull)name{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setName:name];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setSerialNumber:(NSString *_Nonnull)guid
                     sn:(NSString *_Nonnull)sn{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setSerialNumber:sn];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (NSArray<NSNumber *> *_Nonnull)supportedCommands:(NSString *_Nonnull)guid{
    NSMutableArray<NSNumber *>* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result addObjectsFromArray:[sensor supportedCommands]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSArray<NSNumber *> *_Nonnull)supportedFeatures:(NSString *_Nonnull)guid{
    NSMutableArray<NSNumber *>* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result addObjectsFromArray:[sensor supportedFeatures]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSArray<NSDictionary *> *_Nonnull)supportedParameters:(NSString *_Nonnull)guid{
    NSMutableArray<NSDictionary *>* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result addObjectsFromArray:[sensor supportedParameters]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getAmpMode:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getAmpMode];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber * _Nonnull)getSamplingFrequencyResist:(NSString * _Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequencyResist];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}


- (NSNumber *_Nonnull)getSamplingFrequencyFPG:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequencyFPG];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getIrAmplitude:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getIrAmplitude];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getRedAmplitude:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getRedAmplitude];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (void)setIrAmplitude:(NSString *_Nonnull)guid
                   amp:(NTIrAmplitude)amp{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setIrAmplitude:amp];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setRedAmplitude:(NSString *_Nonnull)guid
                    amp:(NTRedAmplitude)amp{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setRedAmplitude:amp];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)pingNeuroSmart:(NSString *_Nonnull)guid
                marker:(NSNumber*_Nonnull)marker
               resolve:(RCTPromiseResolveBlock _Nonnull )resolve
                reject:(RCTPromiseRejectBlock _Nonnull )reject{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor pingNeuroSmart:marker];
        resolve(@"success");
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
        reject(@"pingNeuroSmart", exception.description, nil);
    }
}


- (NSNumber * _Nonnull)isSupportedFilter:(NSString * _Nonnull)guid
                         filter:(NSNumber * _Nonnull)filter{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor isSupportedFilter:filter];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSArray<NSNumber *> *_Nonnull)getSupportedFilters:(NSString *_Nonnull)guid{
    NSMutableArray* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor getSupportedFilters:result];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSArray<NSNumber *> *_Nonnull)getHardwareFilters:(NSString *_Nonnull)guid{
    NSMutableArray* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor getHardwareFilters:result];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getExtSwInput:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getExtSwInput];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getADCInput:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getADCInput];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getStimulatorMAState:(NSString *_Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result setDictionary:[sensor getStimulatorMAState]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getStimulatorParam:(NSString *_Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result setDictionary:[sensor getStimulatorParam]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getMotionAssistantParam:(NSString *_Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor getMotionAssistantParam: result];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getMotionCounterParam:(NSString *_Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        
        [sensor getMotionCounterParam:result];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getMotionCounter:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getMotionCounter];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getColor:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getColor];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getMEMSCalibrateState:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getMEMSCalibrateState];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getSamplingFrequencyResp:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequencyResp];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber * _Nonnull)getSamplingFrequencyEnvelope:(NSString * _Nonnull)guid {
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequencyEnvelope];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getSignalType:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSignalType];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber * _Nonnull)getElectrodeState:(NSString * _Nonnull)guid {
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getElectrodeState];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (void)setHardwareFilters:(NSString *_Nonnull)guid
                   filters:(NSArray *_Nonnull)filters{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setHardwareFilters:filters];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setFirmwareMode:(NSString *_Nonnull)guid
                   mode:(NTSensorFirmwareMode)mode{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setFirmwareMode:mode];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setSamplingFrequency:(NSString *_Nonnull)guid
                          sf:(NTSensorSamplingFrequency)sf{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setSamplingFrequency:sf];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setGain:(NSString *_Nonnull)guid
           gain:(NTSensorGain)gain{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setGain:gain];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setDataOffset:(NSString *_Nonnull)guid
               offset:(NTSensorDataOffset)offset{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setDataOffset:offset];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setExtSwInput:(NSString *_Nonnull)guid
             extSwInp:(NTSensorExternalSwitchInput)extSwInp{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setExtSwInput:extSwInp];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setADCInput:(NSString *_Nonnull)guid
             adcInp:(NTSensorADCInput)adcInp{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setADCInput:adcInp];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setStimulatorParam:(NSString *_Nonnull)guid
                             param:(NTCallibriStimulationParams*_Nonnull)param{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setStimulatorParam:param];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setMotionAssistantParam:(NSString *_Nonnull)guid
                                  param:(NTCallibriMotionAssistantParams*_Nonnull)param{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setMotionAssistantParam:param];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setMotionCounterParam:(NSString *_Nonnull)guid
                                param:(NTCallibriMotionCounterParam*_Nonnull)param{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setMotionCounterParam:param];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setSignalType:(NSString *_Nonnull)guid
                         type:(NTCallibriSignalType)type{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setSignalType:type];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}


- (NSNumber *_Nonnull)getAccSens:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getAccSens];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getGyroSens:(NSString *_Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getGyroSens];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSNumber *_Nonnull)getSamplingFrequencyMEMS:(NSString *_Nonnull)guid{
    NSNumber* result = @0xFF;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSamplingFrequencyMEMS];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (void)setAccSens:(NSString *_Nonnull)guid
           accSens:(NTSensorAccelerometerSensitivity)accSens{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setAccSens:accSens];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setGyroSens:(NSString *_Nonnull)guid
           gyroSens:(NTSensorGyroscopeSensitivity)gyroSens{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setGyroSens:gyroSens];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}


- (NSNumber * _Nonnull)getSurveyId:(NSString * _Nonnull)guid{
    NSNumber* result = @0;
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        result = [sensor getSurveyId];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (NSDictionary *_Nonnull)getAmplifierParam:(NSString * _Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result setDictionary:[sensor getAmplifierParam]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}

- (void)setSurveyId:(NSString *_Nonnull)guid
                             id:(NSNumber *_Nonnull)id{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setSurveyId:id];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}

- (void)setAmplifierParam:(NSString *_Nonnull)guid
                             param:(NTNeuroEEGAmplifierParam*_Nonnull)param{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setAmplifierParam:param];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}


- (NSArray * _Nullable)getSupportedChannels:(NSString * _Nonnull)guid{
    NSMutableArray<NSNumber *>* result = [NSMutableArray new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result addObjectsFromArray:[sensor getSupportedChannels]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}


- (NSDictionary *_Nonnull)getAmplifierParamBB2:(NSString * _Nonnull)guid{
    NSMutableDictionary* result = [NSMutableDictionary new];
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [result setDictionary:[sensor getAmplifierParamBB2]];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
    return result;
}
- (void)setAmplifierParamBB2:(NSString *_Nonnull)guid
                             param:(NTBrainBit2AmplifierParam*_Nonnull)param{
    @try {
        AFullSensor* sensor = [_sensors objectForKey:guid];
        [sensor setAmplifierParamBB2:param];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception.description);
    }
}



@end
