import type { TurboModule } from 'react-native';
import { TurboModuleRegistry } from 'react-native';

export enum SensorState{
  InRange = 0,
  OutOfRange = 1
};
export enum SensorSamplingFrequency{
  FrequencyHz10 = 0,
  FrequencyHz20 = 1,
  FrequencyHz100 = 2,
  FrequencyHz125 = 3,
  FrequencyHz250 = 4,
  FrequencyHz500 = 5,
  FrequencyHz1000 = 6,
  FrequencyHz2000 = 7,
  FrequencyHz4000 = 8,
  FrequencyHz8000 = 9,
  FrequencyHz10000 = 10,
  FrequencyHz12000 = 11,
  FrequencyHz16000 = 12,
  FrequencyHz24000 = 13,
  FrequencyHz32000 = 14,
  FrequencyHz48000 = 15,
  FrequencyHz64000 = 16,
  FrequencyUnsupported = 0xFF
};
export enum SensorGain{
  Gain1 = 0,
  Gain2 = 1,
  Gain3 = 2,
  Gain4 = 3,
  Gain6 = 4,
  Gain8 = 5,
  Gain12 = 6,
  Gain24 = 7,
  Gain5 = 8,
  Gain2x = 9,
  Gain4x = 10,
  GainUnsupported = 11 
};
export enum SensorDataOffset{
  DataOffset0 = 0x00,
  DataOffset1 = 0x01,
  DataOffset2 = 0x02,
  DataOffset3 = 0x03,
  DataOffset4 = 0x04,
  DataOffset5 = 0x05,
  DataOffset6 = 0x06,
  DataOffset7 = 0x07,
  DataOffset8 = 0x08,
  DataOffsetUnsupported = 0xFF
};
export enum SensorExternalSwitchInput{
  ExtSwInElectrodesRespUSB = 0,
  ExtSwInElectrodes = 1,
  ExtSwInUSB = 2,
  ExtSwInRespUSB = 3,
  ExtSwInShort = 4,
  ExtSwInUnknown = 0xFF
};
export enum SensorADCInput{
  Electrodes = 0,
  Short = 1,
  Test = 2,
  Resistance = 3
};

export enum SensorAccelerometerSensitivity{
  AccSens2g = 0,
  AccSens4g = 1,
  AccSens8g = 2,
  AccSens16g = 3,
  AccSensUnsupported = 4
};
export enum SensorGyroscopeSensitivity{
  GyroSens250Grad = 0,
  GyroSens500Grad = 1,
  GyroSens1000Grad = 2,
  GyroSens2000Grad = 3,
  GyroSensUnsupported = 4
};

export enum SensorFirmwareMode{
  ModeBootloader = 0,
  ModeApplication = 1
};

export enum IrAmplitude{ 
  IrAmp0 = 0,
  IrAmp14 = 1,
  IrAmp28 = 2,
  IrAmp42 = 3,
  IrAmp56 = 4,
  IrAmp70 = 5,
  IrAmp84 = 6,
  IrAmp100 = 7,
  IrAmpUnsupported = 0xFF
};
export enum RedAmplitude{
  RedAmp0 = 0,
  RedAmp14 = 1,
  RedAmp28 = 2,
  RedAmp42 = 3,
  RedAmp56 = 4,
  RedAmp70 = 5,
  RedAmp84 = 6,
  RedAmp100 = 7,
  RedAmpUnsupported = 0xFF
};
export type FPGData = {
  PackNum: number
  IrAmplitude: number
  RedAmplitude: number
}


export enum SensorAmpMode{
  AmpModeInvalid = 0,
  AmpModePowerDown = 1,
  AmpModeIdle = 2,
  AmpModeSignal = 3,
  AmpModeResist = 4,
  AmpModeSignalResist = 5,
  AmpModeEnvelope = 6
};

export enum SensorCommand {
  StartSignal = 0,
  StopSignal = 1,
  StartResist = 2,
  StopResist = 3,
  StartMEMS = 4,
  StopMEMS = 5,
  StartRespiration = 6,
  StopRespiration = 7,
  StartCurrentStimulation = 8,
  StopCurrentStimulation = 9,
  EnableMotionAssistant = 10,
  DisableMotionAssistant = 11,
  FindMe = 12,
  StartAngle = 13,
  StopAngle = 14,
  CalibrateMEMS = 15,
  ResetQuaternion = 16,
  StartEnvelope = 17,
  StopEnvelope = 18,
  ResetMotionCounter = 19,
  CalibrateStimulation = 20,
  Idle = 21,
  PowerDown = 22,
  StartFPG = 23,
  StopFPG = 24,
  StartSignalAndResist = 25,
  StopSignalAndResist = 26,
  StartPhotoStimulation = 27,
  StopPhotoStimulation = 28,
  StartAcousticStimulation = 29,
  StopAcousticStimulation = 30,
  FileSystemEnable = 31,
  FileSystemDisable = 32,
  FileSystemStreamClose = 33,
  StartCalibrateSignal = 34,
  StopCalibrateSignal = 35,
	PhotoStimEnable = 36,
	PhotoStimDisable = 37
}
export enum SensorParamAccess{
  Read = 0,
  ReadWrite = 1,
  ReadNotify = 2
}
export enum SensorParameter
{
  Name = 0,
  State = 1,
  Address = 2,
  SerialNumber = 3,
  HardwareFilterState = 4,
  FirmwareMode = 5,
  SamplingFrequency = 6,
  Gain = 7,
  Offset = 8,
  ExternalSwitchState = 9,
  ADCInputState = 10,
  AccelerometerSens = 11,
  GyroscopeSens = 12,
  StimulatorAndMAState = 13,
  StimulatorParamPack = 14,
  MotionAssistantParamPack = 15,
  FirmwareVersion = 16,
  MEMSCalibrationStatus = 17,
  MotionCounterParamPack = 18,
  MotionCounter = 19,
  BattPower = 20,
  SensorFamily = 21,
  SensorMode = 22,
  IrAmplitude = 23,
  RedAmplitude = 24,
  EnvelopeAvgWndSz = 25,
  EnvelopeDecimation = 26,
  SamplingFrequencyResist = 27,
  SamplingFrequencyMEMS = 28,
  SamplingFrequencyFPG = 29,
  Amplifier = 30,
  SensorChannels = 31,
  SamplingFrequencyResp = 32,
  SurveyId = 33,

  ReferentsShort = 36,
  ReferentsGround = 37,
  SamplingFrequencyEnvelope = 38,
  ChannelConfiguration = 39,
  ElectrodeState = 40,
  ChannelResistConfiguration = 41,
  BattVoltage = 42,
  PhotoStimTimeDefer = 43,
  PhotoStimSyncState = 44,
  SensorPhotoStim = 45,
  PhotoStimMode = 46
}

export enum SensorFeature{
  Signal = 0,
  MEMS = 1,
  CurrentStimulator = 2,
  Respiration = 3,
  Resist = 4,
  FPG = 5,
  Envelope = 6,
  PhotoStimulator = 7,
  AcousticStimulator = 8,
  FeatureFlashCard = 9
}
export enum SensorFilter {
  FilterHPFBwhLvl1CutoffFreq1Hz = 0,
  FilterHPFBwhLvl1CutoffFreq5Hz = 1,
  FilterBSFBwhLvl2CutoffFreq45_55Hz = 2,
  FilterBSFBwhLvl2CutoffFreq55_65Hz = 3,
  FilterHPFBwhLvl2CutoffFreq10Hz = 4,
  FilterLPFBwhLvl2CutoffFreq400Hz = 5,
  FilterHPFBwhLvl2CutoffFreq80Hz = 6,
  FilterUnknown = 255
}
export enum SensorFamily {
  SensorFamilyUnknown = 0,
  
  LECallibri = 1,
  LEKolibri = 2,
  
  
  LEBrainBit = 3,
  
  
  LEBrainBitBlack = 4,
  
  
  
  LENeuroEEG = 14,
  
  
  LEBrainBit2 = 18,
  LEBrainBitPro = 19,
  LEBrainBitFlex = 20,
  
}

export type ParameterInfo = {
  Param: SensorParameter
  ParamAccess: SensorParamAccess
}

export type SensorInfo = {
  SensFamily: SensorFamily
  SensModel: number
  Name: string
  Address: string
  SerialNumber: string
  PairingRequired: boolean
  RSSI: number
}
export type SensorVersion = {
  FwMajor: number
  FwMinor: number
  FwPatch: number
  HwMajor: number
  HwMinor: number
  HwPatch: number
  ExtMajor: number
}

export type MEMSData = {
  Accelerometer: Point3D
  Gyroscope: Point3D
}
export type Point3D = {
  X: number
  Y: number
  Z: number
}


export type BrainBitSignalData = {
  PackNum: number
  Marker: number
  O1: number
  O2: number
  T3: number
  T4: number
}
export type BrainBitResistData = {
  O1: number
  O2: number
  T3: number
  T4: number
}


export type CallibriSignalData={
  PackNum : number
  Samples : Array<number>
}
export type CallibriEnvelopeData={
  PackNum : number
  Sample : number
}
export type CallibriRespirationData={
  PackNum : number
  Samples : Array<number>
}
export enum CallibriColorType{
  ColorRed = 0,
  ColorYellow = 1,
  ColorBlue = 2,
  ColorWhite = 3,
  ColorUnknown = 4
};
export enum CallibriSignalType {
  EEG = 0,
  EMG = 1,
  ECG = 2,
  EDA = 3,// GSR
  StrainGaugeBreathing = 4,
  ImpedanceBreathing = 5,
  TenzoBreathing = 6,
  SignalTypeUnknown = 7
};
export type CallibriStimulatorMAState = {
  MAState: CallibriStimulatorState
  StimulatorState: CallibriStimulatorState
}
export type CallibriStimulationParams = {
  Current: number
  PulseWidth: number
  Frequency: number
  StimulusDuration: number
}
export type CallibriMotionAssistantParams = {
  GyroStart: number
  GyroStop: number
  Limb: CallibriMotionAssistantLimb
  MinPauseMs: number
}
export type CallibriMotionCounterParam = {
  InsenseThresholdMG: number
  InsenseThresholdSample: number
}
export type QuaternionData = {
  PackNum : number
  W: number
  X: number
  Y: number
  Z: number
}
export enum CallibriElectrodeState
{
  Normal = 0,
  HighResistance = 1,
  Detached = 2
}
export enum CallibriMotionAssistantLimb {
  RightLeg = 0,
  LeftLeg = 1,
  RightArm = 2,
  LeftArm = 3,
  CallibriMotionAssistantLimbUnsupported = 0xFF
}
export enum CallibriStimulatorState{
  NoParams = 0,
  Disabled = 1,
  Enabled = 2,
  CallibriStimulatorStateUnsupported = 0xFF
}


export type SignalChannelsData = {
  PackNum: number,
  Marker: number,
  Samples: Array<number>,
}
export type EEGChannelInfo = {
  Id: EEGChannelId,
  ChType: EEGChannelType,
  Name: string,
  Num: number,
}
export enum EEGChannelId{
  EEGChannelIdUnknown = 0,
  EEGChannelIdO1 = 1,
  EEGChannelIdP3 = 2,
  EEGChannelIdC3 = 3,
  EEGChannelIdF3 = 4,
  EEGChannelIdFp1 = 5,
  EEGChannelIdT5 = 6,
  EEGChannelIdT3 = 7,
  EEGChannelIdF7 = 8,

  EEGChannelIdF8 = 9,
  EEGChannelIdT4 = 10,
  EEGChannelIdT6 = 11,
  EEGChannelIdFp2 = 12,
  EEGChannelIdF4 = 13,
  EEGChannelIdC4 = 14,
  EEGChannelIdP4 = 15,
  EEGChannelIdO2 = 16,

  EEGChannelIdD1 = 17,
  EEGChannelIdD2 = 18,
  EEGChannelIdOZ = 19,
  EEGChannelIdPZ = 20,
  EEGChannelIdCZ = 21,
  EEGChannelIdFZ = 22,
  EEGChannelIdFpZ = 23,
  EEGChannelIdD3 = 24
};
export enum EEGChannelType{
  A1 = 0,
  A2 = 1,
  Differential = 2,
  Ref = 3
};


export type ResistChannelsData = {
  PackNum: number,
  A1: number,
  A2: number,
  Bias: number,
  Values: Array<number>,
}
export type NeuroEEGAmplifierParam = {
  ReferentResistMesureAllow: boolean,
  Frequency: SensorSamplingFrequency,
  ReferentMode: EEGRefMode,
  ChannelMode: Array<EEGChannelMode>,
  ChannelGain: Array<SensorGain>,
}
export enum EEGRefMode{
  HeadTop = 1,
  A1A2 = 2
};
export enum EEGChannelMode{
  EEGChannelModeOff = 0,
  EEGChannelModeShorted = 1,
  EEGChannelModeSignalResist = 2,
  EEGChannelModeSignal = 3,
  EEGChannelModeTest = 4
};




export enum GenCurrent{
  GenCurr0nA = 0,
	GenCurr6nA = 1,
	GenCurr12nA = 2,
	GenCurr18nA = 3,
	GenCurr24nA = 4,
	GenCurr6uA = 5,
	GenCurr24uA = 6,
	Unsupported = 0xFF
}


export enum BrainBit2ChannelMode{
  ChModeShort = 0,
	ChModeNormal = 1
}
export type BrainBit2AmplifierParam = {
  ChSignalMode: Array<BrainBit2ChannelMode>,
  ChResistUse: Array<boolean>,
  ChGain: Array<SensorGain>,
  Current: GenCurrent
}
export type ResistRefChannelsData = {
  PackNum: number,
  Samples: Array<number>,
  Referents: Array<number>,
}


export interface Spec extends TurboModule {
  // Scanner
  createScanner(filters: Array<SensorFilter>): Promise<string>;
  closeScanner(guid: string): Promise<string>;
  startScan(guid: string): Promise<string>;
  stopScan(guid: string): Promise<string>;
  createSensor(guid: string, sensorInfo: SensorInfo): Promise<string>;
  getSensors(guid: string): Promise<Array<SensorInfo>|string>;

  // Sensor
  closeSensor(guid: string): void;
  connectSensor(guid: string): Promise<string>;
  disconnectSensor(guid: string): Promise<string>;
  supportedFeatures(guid: string): Array<SensorFeature>;
  isSupportedFeature(guid: string, feature: SensorFeature): boolean;
  supportedCommands(guid: string): Array<SensorCommand>;
  isSupportedCommand(guid: string, command: SensorCommand): boolean;
  supportedParameters(guid: string): Array<ParameterInfo>;
  isSupportedParameter(guid: string, parameter: SensorParameter): boolean;
  execCommand(guid: string, command: SensorCommand): Promise<string>;
  
  getName(guid: string): string;
  setName(guid: string, name: string): void;
  getState(guid: string): SensorState;
  getAddress(guid: string): string;
  getSerialNumber(guid: string): string;
  setSerialNumber(guid: string, sn: string): void;
  getBattPower(guid: string): number;
  getSamplingFrequency(guid: string): SensorSamplingFrequency;
  getGain(guid: string): SensorGain;
  getDataOffset(guid: string): SensorDataOffset;
  getFirmwareMode(guid: string): SensorFirmwareMode;
  getVersion(guid: string): SensorVersion;
  getChannelsCount(guid: string): number;
  getSensFamily(guid: string): SensorFamily;

  getAmpMode(guid: string):SensorAmpMode
  getSamplingFrequencyResist(guid: string): SensorSamplingFrequency


  getSamplingFrequencyFPG(guid: string):SensorSamplingFrequency;
  getIrAmplitude(guid: string):IrAmplitude;
  setIrAmplitude(guid: string, amp: IrAmplitude): void;
  getRedAmplitude(guid: string):RedAmplitude;
  setRedAmplitude(guid: string, amp: RedAmplitude): void;


  pingNeuroSmart(guid: string, marker: number): Promise<string>;


  setGain(guid: string, gain: SensorGain):void;


  isSupportedFilter(guid: string, filter: SensorFilter):boolean;

  getSupportedFilters(guid: string):Array<SensorFilter>;
  getHardwareFilters(guid: string):Array<SensorFilter>;
  setHardwareFilters(guid: string, filters: Array<SensorFilter>):void;
  setFirmwareMode(guid: string, mode:SensorFirmwareMode):void;
  setSamplingFrequency(guid: string, sf:SensorSamplingFrequency):void;
  setDataOffset(guid: string, offset: SensorDataOffset):void;
  getExtSwInput(guid: string):SensorExternalSwitchInput;
  setExtSwInput(guid: string, extSwInp: SensorExternalSwitchInput):void;
  getADCInput(guid: string):SensorADCInput;
  setADCInput(guid: string, adcInp: SensorADCInput):void;
  getStimulatorMAState(guid: string):CallibriStimulatorMAState;
  getStimulatorParam(guid: string): CallibriStimulationParams;
  setStimulatorParam(guid: string, param: CallibriStimulationParams):void;
  getMotionAssistantParam(guid: string):CallibriMotionAssistantParams;
  setMotionAssistantParam(guid: string, param: CallibriMotionAssistantParams):void;
  getMotionCounterParam(guid: string):CallibriMotionCounterParam;
  setMotionCounterParam(guid: string, param: CallibriMotionCounterParam):void;
  getMotionCounter(guid: string):number;
  getColor(guid: string):CallibriColorType;
  getMEMSCalibrateState(guid: string):boolean;
  getSamplingFrequencyResp(guid: string):SensorSamplingFrequency;
  getSamplingFrequencyEnvelope(guid: string):SensorSamplingFrequency;
  getSignalType(guid: string):CallibriSignalType;
  setSignalType(guid: string, type: CallibriSignalType):void;
  getElectrodeState(guid: string): CallibriElectrodeState;


  getAccSens(guid: string):SensorAccelerometerSensitivity;
  setAccSens(guid: string, accSens: SensorAccelerometerSensitivity):void;
  getGyroSens(guid: string):SensorGyroscopeSensitivity;
  setGyroSens(guid: string, gyroSens: SensorGyroscopeSensitivity):void;

  getSamplingFrequencyMEMS(guid: string):SensorSamplingFrequency;


  getSupportedChannels(guid: string): Array<EEGChannelInfo>;


  getAmplifierParamBB2(guid: string): BrainBit2AmplifierParam;
  setAmplifierParamBB2(guid: string, param: BrainBit2AmplifierParam): number;



  getSurveyId(guid: string): number;
  setSurveyId(guid: string, id: number): void;
  getAmplifierParam(guid: string): NeuroEEGAmplifierParam;
  setAmplifierParam(guid: string, param: NeuroEEGAmplifierParam): void;


}

export default TurboModuleRegistry.getEnforcing<Spec>('Neurosdk2');
