import { EmitterSubscription, NativeEventEmitter } from 'react-native';
import { Neurosdk2 } from "./ModuleResolver";
import type { SensorInfo } from "./NativeNeurosdk2";
import { SensorFamily }from "./NativeNeurosdk2";

import CallibriSensor from './Sensors/CallibriSensor';


import BrainBitSensor from './Sensors/BrainBitSensor';


import BrainBitBlackSensor from './Sensors/BrainBitBlackSensor';


import NeuroEEGSensor from './Sensors/NeuroEEGSensor';


import BrainBit2Sensor from './Sensors/BrainBit2Sensor';


import type Sensor from './Sensor';

class Scanner
{ 
    private sensorListChanged: EmitterSubscription | undefined;
    private guid: string = ""
    
    async init(filters: Array<SensorFamily>) {
        try {
            this.guid = await Neurosdk2.createScanner(filters);
        }
        catch (err) {
            console.log(err)
        }
    }

    close():Promise<string>
    {
        return Neurosdk2.closeScanner(this.guid);
    }

    createSensor(sensorInfo: SensorInfo): Promise<Sensor> 
    {
        return new Promise<Sensor>(async (resolve, reject) => {
            Neurosdk2.createSensor(this.guid, sensorInfo)
                .then((guid: string)=>{
                    
                    if(sensorInfo.SensFamily == SensorFamily.LECallibri || sensorInfo.SensFamily == SensorFamily.LEKolibri)
                        resolve(new CallibriSensor(guid));
                    
                    
                    if(sensorInfo.SensFamily == SensorFamily.LEBrainBit)
                        resolve(new BrainBitSensor(guid));
                    
                    
                    if(sensorInfo.SensFamily == SensorFamily.LEBrainBitBlack)
                        resolve(new BrainBitBlackSensor(guid));
                    
                    
                    if(sensorInfo.SensFamily == SensorFamily.LENeuroEEG)
                        resolve(new NeuroEEGSensor(guid));
                    
                    
                    if(sensorInfo.SensFamily == SensorFamily.LEBrainBit2 || 
                        sensorInfo.SensFamily == SensorFamily.LEBrainBitPro || 
                        sensorInfo.SensFamily == SensorFamily.LEBrainBitFlex)
                        resolve(new BrainBit2Sensor(guid));
                    
                    
                    })
                .catch((c: string)=>{console.log(c); reject(c);})
        });
    }
    sensors(): Promise<Array<SensorInfo>> {
        return Neurosdk2.getSensors(this.guid);
    }
    start(): Promise<string> {
        return Neurosdk2.startScan(this.guid);
    }
    stop(): Promise<string> {
        return Neurosdk2.stopScan(this.guid);
    }
    AddSensorListChanged(callback:(sensorFounded:SensorInfo[]) => void)
    {
        var sensorListChangedEmitter = new NativeEventEmitter(Neurosdk2);
        this.sensorListChanged = sensorListChangedEmitter.addListener(
            Neurosdk2.SENSOR_LIST_CHANGED_EVENT_NAME,
            (list) => {
                // if (list.GUID == undefined) { callback(list) }
                if (this.guid == list.GUID) { callback(list.data) }  
            }
        );
    }
    RemoveSensorListChanged(){
        if(this.sensorListChanged != undefined){  
            this.sensorListChanged.remove()  
        }   
    }
}
export default Scanner;