import type { EmitterSubscription } from "react-native";
import Sensor from "../Sensor";
import type { SignalChannelsData, ResistRefChannelsData, BrainBit2AmplifierParam, EEGChannelInfo, MEMSData, SensorAmpMode, FPGData, SensorSamplingFrequency, IrAmplitude, RedAmplitude, SensorAccelerometerSensitivity, SensorGyroscopeSensitivity } from "../NativeNeurosdk2";
import { Neurosdk2 } from "../ModuleResolver";

export default class BrainBit2Sensor extends Sensor {

    private resistDataReceived: EmitterSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;

    private AMPModeChanged: EmitterSubscription | undefined;
    private MEMSDataChanged: EmitterSubscription | undefined;
    private neuroSmartFPGChanged: EmitterSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }

    static getMaxChCount(): number{
        return 8;
    }

    getSupportedChannels(): Array<EEGChannelInfo>{
        return Neurosdk2.getSupportedChannels(this.guid);
    }

    getAmplifierParam(): BrainBit2AmplifierParam{
        return Neurosdk2.getAmplifierParamBB2(this.guid);
    }
    setAmplifierParam(param: BrainBit2AmplifierParam){
        Neurosdk2.setAmplifierParamBB2(this.guid, param);
    }

    getSamplingFrequencyMEMS(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyMEMS(this.guid);
    }

    getSamplingFrequencyFPG(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyFPG(this.guid);
    }

    getSamplingFrequencyResist(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyResist(this.guid);
    }

    getIrAmplitude(): IrAmplitude {
        return Neurosdk2.getIrAmplitude(this.guid);
    }
    setIrAmplitude(amp: IrAmplitude) {
        Neurosdk2.setIrAmplitude(this.guid, amp)
    }

    getRedAmplitude(): RedAmplitude {
        return Neurosdk2.getRedAmplitude(this.guid);
    }
    setRedAmplitude(amp: RedAmplitude) {
        Neurosdk2.setRedAmplitude(this.guid, amp);
    }

    getAmpMode(): SensorAmpMode {
        return Neurosdk2.getAmpMode(this.guid);
    }

    getAccSens(): SensorAccelerometerSensitivity {
        return Neurosdk2.getAccSens(this.guid);
    }
    setAccSens(accSens: SensorAccelerometerSensitivity) {
        Neurosdk2.setAccSens(this.guid, accSens);
    }

    getGyroSens(): SensorGyroscopeSensitivity {
        return Neurosdk2.getGyroSens(this.guid);
    }
    setGyroSens(gyroSens: SensorGyroscopeSensitivity) {
        Neurosdk2.setGyroSens(this.guid, gyroSens);
    }

    pingNeuroSmart(marker: number): Promise<string> {
        return Neurosdk2.pingNeuroSmart(this.guid, marker);
    }

    AddAMPModeChanged(callback: (AMPMode: SensorAmpMode) => void) {
        this.AMPModeChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.AMPMODE_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveAMPModeChanged() {
        if (this.AMPModeChanged != undefined)
            this.AMPModeChanged.remove();
    }

    AddMEMSDataChanged(callback: (data: Array<MEMSData>) => void) {
        this.MEMSDataChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.MEMSDATA_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveMEMSDataChanged() {
        if (this.MEMSDataChanged != undefined)
            this.MEMSDataChanged.remove();
    }

    AddNeuroSmartFPGDataChanged(callback: (FPGData: Array<FPGData>) => void) {
        this.neuroSmartFPGChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME,
            (signalData) => { if (signalData.GUID === this.guid) callback(signalData.data) }
        );
    }
    RemoveNeuroSmartFPGDataChanged() {
        if (this.neuroSmartFPGChanged != undefined)
            this.neuroSmartFPGChanged.remove();
    }

    AddSignalReceived(callback: (signalData: Array<SignalChannelsData>) => void) {
        this.signalDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT2_SIGNAL_RECEIVED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveSignalReceived() {
        if (this.signalDataReceived != undefined)
            this.signalDataReceived.remove();
    }
    AddResistanceReceived(callback: (resistanceData: Array<ResistRefChannelsData>) => void) {
        this.resistDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT2_RESIST_RECEIVED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveResistanceReceived() {
        if (this.resistDataReceived != undefined)
            this.resistDataReceived.remove();
    }
}