import type { EmitterSubscription } from "react-native";
import BrainBitSensor from "./BrainBitSensor";
import type { MEMSData, SensorAmpMode, FPGData, SensorSamplingFrequency, IrAmplitude, RedAmplitude, SensorAccelerometerSensitivity, SensorGyroscopeSensitivity } from "../NativeNeurosdk2";
import { Neurosdk2 } from "../ModuleResolver";

export default class BrainBitBlackSensor extends BrainBitSensor {

    private AMPModeChanged: EmitterSubscription | undefined;
    private MEMSDataChanged: EmitterSubscription | undefined;
    private neuroSmartFPGChanged: EmitterSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }

    getSamplingFrequencyMEMS(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyMEMS(this.guid);
    }

    getSamplingFrequencyFPG(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyFPG(this.guid);
    }

    getSamplingFrequencyResist(): SensorSamplingFrequency {
        return Neurosdk2.getSamplingFrequencyResist(this.guid);
    }

    getIrAmplitude(): IrAmplitude {
        return Neurosdk2.getIrAmplitude(this.guid);
    }
    setIrAmplitude(amp: IrAmplitude) {
        Neurosdk2.setIrAmplitude(this.guid, amp)
    }

    getRedAmplitude(): RedAmplitude {
        return Neurosdk2.getRedAmplitude(this.guid);
    }
    setRedAmplitude(amp: RedAmplitude) {
        Neurosdk2.setRedAmplitude(this.guid, amp);
    }

    getAmpMode(): SensorAmpMode {
        return Neurosdk2.getAmpMode(this.guid);
    }

    getAccSens(): SensorAccelerometerSensitivity {
        return Neurosdk2.getAccSens(this.guid);
    }
    setAccSens(accSens: SensorAccelerometerSensitivity) {
        Neurosdk2.setAccSens(this.guid, accSens);
    }

    getGyroSens(): SensorGyroscopeSensitivity {
        return Neurosdk2.getGyroSens(this.guid);
    }
    setGyroSens(gyroSens: SensorGyroscopeSensitivity) {
        Neurosdk2.setGyroSens(this.guid, gyroSens);
    }

    pingNeuroSmart(marker: number): Promise<string> {
        return Neurosdk2.pingNeuroSmart(this.guid, marker);
    }



    AddAMPModeChanged(callback: (AMPMode: SensorAmpMode) => void) {
        this.AMPModeChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.AMPMODE_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveAMPModeChanged() {
        if (this.AMPModeChanged != undefined)
            this.AMPModeChanged.remove();
    }

    AddMEMSDataChanged(callback: (data: Array<MEMSData>) => void) {
        this.MEMSDataChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.MEMSDATA_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveMEMSDataChanged() {
        if (this.MEMSDataChanged != undefined)
            this.MEMSDataChanged.remove();
    }

    AddNeuroSmartFPGDataChanged(callback: (FPGData: Array<FPGData>) => void) {
        this.neuroSmartFPGChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.NEUROSMART_FPGDATA_CHANGED_EVENT_NAME,
            (signalData) => { if (signalData.GUID === this.guid) callback(signalData.data) }
        );
    }
    RemoveNeuroSmartFPGDataChanged() {
        if (this.neuroSmartFPGChanged != undefined)
            this.neuroSmartFPGChanged.remove();
    }

}
