import type { EmitterSubscription } from "react-native";
import Sensor from "../Sensor";
import type { BrainBitResistData, BrainBitSignalData, SensorGain } from "../NativeNeurosdk2";
import { Neurosdk2 } from "../ModuleResolver";

export default class BrainBitSensor extends Sensor {

    private resistDataReceived: EmitterSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }

    AddSignalReceived(callback: (signalData: Array<BrainBitSignalData>) => void) {
        this.signalDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT_SIGNAL_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveSignalReceived() {
        if (this.signalDataReceived != undefined)
            this.signalDataReceived.remove();
    }
    AddResistanceReceived(callback: (resistanceData: BrainBitResistData) => void) {
        this.resistDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.BRAINBIT_RESISTANCE_CHANGED_EVENT_NAME,
            (data) => { if (data.GUID === this.guid) callback(data.data) }
        );
    }
    RemoveResistanceReceived() {
        if (this.resistDataReceived != undefined)
            this.resistDataReceived.remove();
    }

    getGain(): SensorGain {
        return Neurosdk2.getGain(this.guid);
    }

    setGain(gain: SensorGain){
      Neurosdk2.setGain(this.guid, gain);
    }

}