import Sensor from "../Sensor";
import type { EmitterSubscription } from "react-native";
import { Neurosdk2 } from "../ModuleResolver";
import type { CallibriColorType, CallibriElectrodeState, CallibriEnvelopeData, CallibriMotionAssistantParams, CallibriMotionCounterParam, CallibriRespirationData, CallibriSignalData, CallibriSignalType, CallibriStimulationParams, CallibriStimulatorMAState, MEMSData, QuaternionData, SensorADCInput, SensorAccelerometerSensitivity, SensorDataOffset, SensorExternalSwitchInput, SensorFilter, SensorFirmwareMode, SensorGain, SensorGyroscopeSensitivity, SensorSamplingFrequency } from "../NativeNeurosdk2";

export default class CallibriSensor extends Sensor {
 
    private MEMSDataReceived: EmitterSubscription | undefined;
    private signalReceived: EmitterSubscription | undefined;
    private electrodeStateChanged: EmitterSubscription | undefined;
    private envelopeDataReceived: EmitterSubscription | undefined;
    private respirationReceived: EmitterSubscription | undefined;
    private quaternionDataReceived: EmitterSubscription | undefined;

    constructor(guid: string) {
        super(guid)
    }
    
    getGain(): SensorGain {
      return Neurosdk2.getGain(this.guid);
    }
    setGain(gain: SensorGain){
      Neurosdk2.setGain(this.guid, gain);
    }
  
    getGyroSens(): SensorGyroscopeSensitivity{
      return Neurosdk2.getGyroSens(this.guid);
    }
    setGyroSens(gyroSens: SensorGyroscopeSensitivity){
      Neurosdk2.setGyroSens(this.guid, gyroSens);
    }
    getSamplingFrequencyMEMS(): SensorSamplingFrequency{
      return Neurosdk2.getSamplingFrequencyMEMS(this.guid);
    }
    getSamplingFrequencyResp(): SensorSamplingFrequency{
      return Neurosdk2.getSamplingFrequencyResp(this.guid);    
    }  
    getSamplingFrequencyEnvelope(): SensorSamplingFrequency{
      return Neurosdk2.getSamplingFrequencyEnvelope(this.guid);    
    }
    getAccSens(): SensorAccelerometerSensitivity{
      return Neurosdk2.getAccSens(this.guid);    
    } 
    setAccSens(accSens: SensorAccelerometerSensitivity){
      Neurosdk2.setAccSens(this.guid, accSens);
    }
  
    getStimulatorMAState(): CallibriStimulatorMAState {
      return Neurosdk2.getStimulatorMAState(this.guid);
    }
    getStimulatorParam(): CallibriStimulationParams {
      return Neurosdk2.getStimulatorParam(this.guid);
    }
    setStimulatorParam(stimulatorParamCallibri: CallibriStimulationParams) {
      Neurosdk2.setStimulatorParam(this.guid, stimulatorParamCallibri);
    }
    setName(name: string){
      Neurosdk2.setName(this.guid, name);
    }
    setFirmwareMode(firmwareMode: SensorFirmwareMode){
      Neurosdk2.setFirmwareMode(this.guid, firmwareMode);
    }
    isSupportedFilter(filter: SensorFilter): boolean {
      return Neurosdk2.isSupportedFilter(this.guid, filter);
    }
    getSupportedFilters(): Array<SensorFilter> {
      return Neurosdk2.getSupportedFilters(this.guid);
    }
    getHardwareFilters(): Array<SensorFilter> {
      return Neurosdk2.getHardwareFilters(this.guid);
    }
    setHardwareFilters(hardwareFilter: Array<SensorFilter>){
      Neurosdk2.setHardwareFilters(this.guid, hardwareFilter);
    }
    getExtSwInput(): SensorExternalSwitchInput {
      return Neurosdk2.getExtSwInput(this.guid);
    }
    setExtSwInput(extSwInput: SensorExternalSwitchInput){
      Neurosdk2.setExtSwInput(this.guid, extSwInput);
    }
    getADCInput(): SensorADCInput{
      return Neurosdk2.getADCInput(this.guid);    
    } 
    setADCInput(ADCInput: SensorADCInput){
      Neurosdk2.setADCInput(this.guid, ADCInput);
    }
    setDataOffset(dataOffset: SensorDataOffset){
      Neurosdk2.setDataOffset(this.guid, dataOffset);
    }
    setSamplingFrequency(samplingFrequency: SensorSamplingFrequency) {
      Neurosdk2.setSamplingFrequency(this.guid, samplingFrequency);
    }
    getMotionAssistantParam(): CallibriMotionAssistantParams {
      return Neurosdk2.getMotionAssistantParam(this.guid);
    }
    setMotionAssistantParam(motionAssistantParamCallibri: CallibriMotionAssistantParams){
      Neurosdk2.setMotionAssistantParam(this.guid, motionAssistantParamCallibri);
    }
    getMotionCounterParam(): CallibriMotionCounterParam {
      return Neurosdk2.getMotionCounterParam(this.guid);
    }
    setMotionCounterParam(motionCounterParamCallibri: CallibriMotionCounterParam){
      Neurosdk2.setMotionCounterParam(this.guid, motionCounterParamCallibri);
    }
    getMotionCounter(): number{
      return Neurosdk2.getMotionCounter(this.guid);    
    } 
    getColor(): CallibriColorType{
      return Neurosdk2.getColor(this.guid);    
    }
    getSignalType(): CallibriSignalType{
      return Neurosdk2.getSignalType(this.guid);
    }
    setSignalType(signalType: CallibriSignalType){
      Neurosdk2.setSignalType(this.guid, signalType);
    }
    getElectrodeState(): CallibriElectrodeState {
      return Neurosdk2.getElectrodeState(this.guid)
    }
    getMEMSCalibrateState(): boolean{
      return Neurosdk2.getMEMSCalibrateState(this.guid)
    }
  

    AddMMEMSDataReceived(callback:(data:Array<MEMSData>) => void){
        this.MEMSDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.MEMSDATA_CHANGED_EVENT_NAME,
          (data) => { if(data.GUID === this.guid) callback(data.data)}
        );
      }
      RemoveMEMSDataReceived(){
        if(this.MEMSDataReceived != undefined)
            this.MEMSDataReceived.remove();
      } 
    
      AddQuaternionDataReceived(callback:(data:Array<QuaternionData>) => void){
        this.quaternionDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.QUATERNIONDATA_CHANGED_EVENT_NAME,
          (data) => { if(data.GUID === this.guid) callback(data)}
        );
      }
      RemoveQuaternionDataReceived(){
        if(this.quaternionDataReceived != undefined)
            this.quaternionDataReceived.remove();
      } 
      
      AddSignalReceived(callback:(signalData:Array<CallibriSignalData>)=>void){
        this.signalReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.CALLIBRI_SIGNAL_DATA_EVENT_NAME,
          (signalData) => { if(signalData.GUID === this.guid) callback(signalData.data) }
        );
      }
      RemoveSignalReceived(){
        if(this.signalReceived != undefined)
            this.signalReceived.remove();
      } 
      AddElectrodeStateChanged(callback:(electrodeState:CallibriElectrodeState)=>void){
        this.electrodeStateChanged = this.nativeEventEmitter.addListener(
            Neurosdk2.CALLIBRI_ELECTRODE_STATE_EVENT_NAME,
          (state) => {if(state.GUID === this.guid) callback(state.data.CallibriElectrodeState)}
        );
      }
      RemoveElectrodeStateChanged(){
        if(this.electrodeStateChanged != undefined)
            this.electrodeStateChanged.remove();
      } 
      AddEnvelopeDataChanged(callback:(envelopeData:Array<CallibriEnvelopeData>)=>void){
        this.envelopeDataReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.CALLIBRI_ENVELOPE_DATA_EVENT_NAME,
          (data) => {if(data.GUID === this.guid)  callback(data.data)}
        );
      }
      RemoveEnvelopeDataChanged(){
        if(this.envelopeDataReceived != undefined)
            this.envelopeDataReceived.remove();
      } 
      AddRespirationChanged(callback:(respirationData:Array<CallibriRespirationData>)=>void){
        this.respirationReceived = this.nativeEventEmitter.addListener(
            Neurosdk2.CALLIBRI_RESPIRATION_DATA_EVENT_NAME,
          (respirationData) => {if(respirationData.GUID === this.guid)  callback(respirationData.data)}
        );
      }
      RemoveRespirationChanged(){
        if(this.respirationReceived != undefined)
            this.respirationReceived.remove();
      } 

}