import Sensor from "../Sensor";
import type { EmitterSubscription } from "react-native";
import { Neurosdk2 } from "../ModuleResolver";
import type { SensorAmpMode, SensorSamplingFrequency, EEGChannelInfo, NeuroEEGAmplifierParam, SignalChannelsData, ResistChannelsData,

} from "../NativeNeurosdk2";

export default class NeuroEEGSensor extends Sensor {

    private ampModeChanged: EmitterSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;
    private resistDataReceived: EmitterSubscription | undefined;
    private signalResistReceived: EmitterSubscription | undefined;


    constructor(guid: string) {
      super(guid)
    }

    static getMaxChCount(): number{
      return 24;
    }

    getSurveyId(): number{
      return Neurosdk2.getSurveyId(this.guid);
    }
    setSurveyId(id: number){
      Neurosdk2.setSurveyId(this.guid, id);
    }

    getAmpMode(): SensorAmpMode{
      return Neurosdk2.getAmpMode(this.guid);
    }

    getSupportedChannels(): Array<EEGChannelInfo>{
      return Neurosdk2.getSupportedChannels(this.guid);
    }

    getAmplifierParam(): NeuroEEGAmplifierParam{
      return Neurosdk2.getAmplifierParam(this.guid);
    }
    setAmplifierParam(param: NeuroEEGAmplifierParam){
      Neurosdk2.setAmplifierParam(this.guid, param);
    }

    getSamplingFrequencyResist(): SensorSamplingFrequency{
      return Neurosdk2.getSamplingFrequencyResist(this.guid);
    }

    AddAmpModeChanged(callback:(ampMode:SensorAmpMode)=>void){
      this.ampModeChanged = this.nativeEventEmitter.addListener(
          Neurosdk2.AMPMODE_CHANGED_EVENT_NAME,
        (ampMode) => { if(ampMode.GUID === this.guid) callback(ampMode.data)}
      );
    }
    RemoveAmpModeChanged(){
      if(this.ampModeChanged != undefined)
        this.ampModeChanged.remove();
    }

    AddSignalDataReceived(callback:(signalData:Array<SignalChannelsData>)=>void){
      this.signalDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME,
        (signalData) => { if(signalData.GUID === this.guid) callback(signalData.data)}
      );
    }
    RemoveSignalDataReceived(){
      if(this.signalDataReceived != undefined)
        this.signalDataReceived.remove();
    }

    AddResistDataReceived(callback:(resistData:Array<ResistChannelsData>)=>void){
      this.resistDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_RESIST_DATA_EVENT_NAME,
        (resistData) => { if(resistData.GUID === this.guid) callback(resistData.data)}
      );
    }
    RemoveResistDataReceived(){
      if(this.resistDataReceived != undefined)
        this.resistDataReceived.remove();
    }

    AddSignalResistReceived(callback:(signalData:Array<SignalChannelsData>, resistData:Array<ResistChannelsData>)=>void){
      this.signalResistReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME,
        (data) => { if(data.GUID === this.guid) callback(data.data.signalData, data.data.resistData)}
      );
    }
    RemoveSignalResistReceived(){
      if(this.signalResistReceived != undefined)
        this.signalResistReceived.remove();
    }

}